
public class AppletMenu : Gtk.Menu {

    private Gtk.ImageMenuItem newTask;
    private Gtk.ImageMenuItem newNote;
    private Gtk.ImageMenuItem showMainWindow;
    private Gtk.Widget ourParent;
    private IBackend backend;

	private Logger log;

    construct {
        newTask = new Gtk.ImageMenuItem.with_mnemonic ( _("New _Task") );
        newTask.image = new Gtk.Image.from_pixbuf(Utils.UI.getPixbuf("/icons/16x16/actions/document-new.png"));

        newNote = new Gtk.ImageMenuItem.with_mnemonic ( _("New _Note") );
        newNote.image = new Gtk.Image.from_pixbuf(Utils.UI.getPixbuf("/icons/16x16/actions/note-new.png"));

        showMainWindow = new Gtk.ImageMenuItem.with_mnemonic ( _("Show _All") );
        showMainWindow.image = new Gtk.Image.from_pixbuf(Utils.UI.getPixbuf("/icons/16x16/emotes/face-monkey.png"));

		newTask.always_show_image = true;
        newNote.always_show_image = true;
        showMainWindow.always_show_image = true;

		log = Logger.getLogger("AppletMenu");
    }
    
    public AppletMenu (IBackend aBackend, Gtk.Widget aParent) {
        ourParent =  aParent.parent;
        backend =  aBackend;
		this.attach_to_widget(ourParent, (sender, menuItem) => {});
        this.append (newTask);
        this.append (newNote);
        this.append (showMainWindow);

		newTask.activate += (obj) => {
			Utils.UI.addTask((this.parent as LucruriGNOMEApplet).backend);
		};

		newNote.activate += (obj) => {
			Utils.UI.addNote((this.parent as LucruriGNOMEApplet).backend);
		};

		showMainWindow.activate += (obj) => {
			LucruriGNOMEApplet applet = (this.parent as LucruriGNOMEApplet);
			//FIXME flickr
			//if the window is on the same window, dont show and hide it... just raide it
			MainWindow mainWindow = applet.mainWindow;
			mainWindow.hide_all ();
			mainWindow.show_all ();
		};

        Gtk.SeparatorMenuItem separator = new Gtk.SeparatorMenuItem();
        this.append (separator);
		fill();
    }

    public void pop() {


        return;
    }


    private void fill () {
        foreach (ICategory category in backend.categories) {
            if (! category.hide) {
				bool empty = true;
				foreach ( ITask task in backend.tasks ) {
					if (!task.done && !task.isDeleted &&
						(task.categoryID == category.id) &&
						(task.showFrom.mktime() < time_t())
						) {
						empty = false;
						TaskMenuItem taskItem = new TaskMenuItem ( task );
						taskItem.always_show_image = true;
						this.append (taskItem);
					}
				}
                if (!empty) {
            		Gtk.SeparatorMenuItem separator = new Gtk.SeparatorMenuItem();
            		this.append (separator);
				}
            }
        }

		if (backend.notes.size > 0){

		    int i = 0;
		    Utils.Backend.sortNotesByUpdatedAtDesc( backend.notes );
		    foreach (INote note in backend.notes) {
				if (!note.isDeleted && ((i < 10) || note.sticky)) {
				    NoteMenuItem noteItem = new NoteMenuItem(note);
					noteItem.always_show_image = true;
					this.append (noteItem);
				}
				i++;
		    }
		}
    }

	public override void deactivate () {
		detach();
		base.deactivate();
	}
}


