using Gtk;

public class CalendarView : Gtk.ScrolledWindow {

	private Logger log;
	private IBackend backend;
	private List<TasksGroup> groups;

    private Gtk.Widget editArea;

	construct {
		log = Logger.getLogger("CalendarView");
	}

    public CalendarView (IBackend aBackend, Gtk.Widget anEditArea)
    {
		backend = aBackend;
		editArea = anEditArea;

		groups = new List<TasksGroup>();
        Gtk.VBox box = new Gtk.VBox(false,0);

        TasksGroup tasksGroup;

        tasksGroup = new TasksGroup(_("Due today"),TasksGroup.groupType.CALENDAR, TasksGroup.CALENDAR_TODAY, backend, this.editArea);
		groups.append(tasksGroup);
        box.pack_start(tasksGroup,false,false,0);

        tasksGroup = new TasksGroup(_("Due in rest of this week"), TasksGroup.groupType.CALENDAR, TasksGroup.CALENDAR_WEEK_REST, backend, this.editArea);
		groups.append(tasksGroup);
        box.pack_start(tasksGroup,false,false,0);

        tasksGroup = new TasksGroup(_("Due next week"), TasksGroup.groupType.CALENDAR, TasksGroup.CALENDAR_WEEK_NEXT, backend, this.editArea);
		groups.append(tasksGroup);
        box.pack_start(tasksGroup,false,false,0);

        tasksGroup = new TasksGroup(_("Due in rest of this month"), TasksGroup.groupType.CALENDAR, TasksGroup.CALENDAR_MONTH_REST, backend, this.editArea);
		groups.append(tasksGroup);
        box.pack_start(tasksGroup,false,false,0);

        tasksGroup = new TasksGroup(_("Due in the next month and later"), TasksGroup.groupType.CALENDAR, TasksGroup.CALENDAR_LATER, backend, this.editArea);
		groups.append(tasksGroup);
        box.pack_start(tasksGroup,false,false,0);

        //add end spacer
        Label endSpacer = new Label("");
        box.pack_start(endSpacer,true,true,0);
        
        Adjustment hScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        Adjustment vScrollAdjustment = new Adjustment (0,0,0,0,0,0);
		set_hadjustment(hScrollAdjustment);
		set_vadjustment(vScrollAdjustment);
        hscrollbar_policy = PolicyType.NEVER;
        vscrollbar_policy = PolicyType.AUTOMATIC;

		Gtk.EventBox evBox = new Gtk.EventBox();

		Gdk.Color backgroundColor = getBackgroundColor ();
		evBox.modify_bg (StateType.NORMAL, backgroundColor);
		evBox.modify_base (StateType.NORMAL, backgroundColor);
		evBox.add(box);
		
        add_with_viewport(evBox);
    }

	public void update() {
		foreach (TasksGroup group in groups) {
			group.update();
		}
	}

    private Gdk.Color getBackgroundColor () {
		Gtk.Style style = Gtk.rc_get_style (this);
		Gdk.Color color = style.base [(int)StateType.NORMAL];
		return color;
    }

}
