using Gtk;
public class CategoriesTreeView : Gtk.TreeView {

	public static const string ALL_CATEGORIES = "-1";
	public static const string UNFILED = "0";

	public string currentFilterRule;
	public Gtk.TreeModelFilter targetModel;
	public bool editable;

	public Gtk.CellRendererText nameRenderer;

	private Logger log;
	construct {
		editable = true;
		log = Logger.getLogger("CategoriesTreeView");
	}

	public CategoriesTreeView (bool edit) {

		rules_hint = false;
		headers_visible = true;
		hover_selection = false;

		if (edit) {
			editable = true;
			reorderable = true;
		} else {
			editable = false;
			reorderable = false;
		}


		get_selection().set_mode(Gtk.SelectionMode.BROWSE);

        Gtk.TreeViewColumn column;

		//
		// Categories Name Column
		//
		column = new Gtk.TreeViewColumn ();
		// Title for Column
		column.title = _("Categories");
		column.sizing = Gtk.TreeViewColumnSizing.AUTOSIZE;
		column.expand = true;
		column.resizable = true;

		Gtk.CellRendererPixbuf editRenderer = new Gtk.CellRendererPixbuf ();
		column.pack_start (editRenderer, false);
		column.set_cell_data_func (
			editRenderer,
			(cell_layout, cell, model, iter) =>	{
				Gtk.CellRendererPixbuf crp = cell as Gtk.CellRendererPixbuf;
        		ICategory category;
				model.get (iter, 0, out category);
				if (category.id == this.ALL_CATEGORIES) {
					crp.pixbuf = Utils.UI.getPixbuf ("/icons/22x22/actions/filter-note-all.png" );
				} else {
        			crp.pixbuf = Utils.UI.getPixbuf ("/icons/22x22/actions/note-full.png" );
				}
    		}
			);


		
		nameRenderer = new Gtk.CellRendererText ();
		column.pack_start (nameRenderer, true);
		column.set_cell_data_func (
			nameRenderer,
			(cell_layout, cell, model, iter) =>	{
				Gtk.CellRendererText crt = cell as Gtk.CellRendererText;
				crt.ellipsize = Pango.EllipsizeMode.END;

        		ICategory category;
				model.get (iter, 0, out category);
				crt.text = category.name;
				crt.markup = category.name;
    		}
			);
		nameRenderer.editable = true;
		append_column (column);

		column = new TreeViewColumn ();
		// Title for Completed/Checkbox Column
		column.title =  "Hide";
		column.sizing = TreeViewColumnSizing.AUTOSIZE;
		column.resizable = false;
		column.clickable = true;

		Gtk.CellRendererToggle hideRenderer = new Gtk.CellRendererToggle ();
		hideRenderer.toggled += (sender, path) => {
			Gtk.TreeIter iter;
			ICategory category = null;
			Gtk.TreePath aPath = new Gtk.TreePath.from_string( path );

			if (!this.model.get_iter (out iter, aPath))
				return; // Do nothing
			this.model.get (iter, 0, out category);
			if ( category == null )
				return;
			category.hide = (category.hide) ? false : true;
			category.update();			
		};
		column.pack_start (hideRenderer, false);
		column.set_cell_data_func (
			hideRenderer,
			(cell_layout, cell, model, iter) =>	{
				Gtk.CellRendererToggle crt = cell as Gtk.CellRendererToggle;
        		ICategory? category;
				model.get (iter, 0, out category);
				if (category == null) return;
				crt.active = category.hide ? true: false;
    		}
			);
		append_column (column);


	}

	public ICategory? getSelectedCategory() {
		Gtk.TreePath path;
		Gtk.TreeViewColumn column;
		get_cursor (out path, out column);
		if (path == null) return null;

        Gtk.TreeIter iter;
        ICategory category;
        if (!model.get_iter (out iter, path))  return null; // Do nothing
	    model.get (iter, 0, out category);	
        return category;
	}


	public override bool button_press_event (Gdk.EventButton eventButton) {
        Gtk.TreePath path;
		Gtk.TreeViewColumn column;

		if (targetModel == null)
			return base.button_press_event(eventButton);		

		int cell_x;
		int cell_y;
		get_path_at_pos (
			(int)eventButton.x, (int)eventButton.y, 
			out path, out column, out cell_x, out cell_y
		);
		if (path == null) return false; // Do nothing
        Gtk.TreeIter iter;
        ICategory category;
        if (!model.get_iter (out iter, path))  return false; // Do nothing
	    model.get (iter, 0, out category);	
        if ( category == null ) return false;	 // if error .. do nothing
		currentFilterRule = category.id;
		targetModel.refilter();
		if ((category.id == ALL_CATEGORIES)||
			(editable == false) 
			) { 
			set_cursor(path, null, false);
			return true;
		} else { 
			//set_cursor(path, null, true);		
			return base.button_press_event(eventButton);
		}
    }



}
