public class CouchDBBackend : GLib.Object, IBackend {

    /*properties */
    public Gee.ArrayList<ITask> tasks { get; set; }
    public Gee.ArrayList<ICategory> categories { get; set; }
    public Gee.ArrayList<IProject> projects { get; set; }
    public Gee.ArrayList<INote> notes { get; set; }

    private const string _name = _("CouchDB Backend");
    public  string name { get {
        //Name of the backend
        return _name; 
        }
	}

    public bool isConfigured { get { return true; } }

    private string _username;
    public string username { set {} }
    private string _password;
    public string password { set {} }
    private string _uri;
    public string uri { set {} }

	private IBackend.voidFunc updateUIFunc;
	private Logger log;

	private static const string DB_NOTES = "notes";
	private static const string DB_TASKS = "tasks";
	private static const string DB_LUCRURI = "lucruri";

    private static const string DOCUMENT_CONFIG = "config";

    private static const string FIELD_RECORD_TYPE = "record_type";
    private static const string FIELD_RECORD_TYPE_VERSION = "record_type_version";
    private static const string FIELD_LUCRURI = "Lucruri";
	private static const string FIELD_NAME = "title";
	private static const string FIELD_TEXT = "content";
	private static const string FIELD_STICKY = "sticky";
	private static const string FIELD_DONE = "done";
	private static const string FIELD_HIDE = "hide";
	private static const string FIELD_PROJECT_ID = "project-id";
	private static const string FIELD_CATEGORY_ID = "category-id";
	private static const string FIELD_CREATED_AT = "create_date";
	private static const string FIELD_UPDATED_AT = "last_change_date";
	private static const string FIELD_COMPLETED_AT = "completed_date";
	private static const string FIELD_DUE_AT = "due_date";
	private static const string FIELD_SHOW_FROM = "show_from_date";
    private static const string FIELD_POSITION = "position";
    private static const string FIELD_HEIGHT = "height";
    private static const string FIELD_WIDTH = "width";
    private static const string STRING_ANNOTATIONS = "{\"Lucruri\": {}}";
    private static const string RECORD_TYPE_NOTE = 
        "http://www.freedesktop.org/wiki/Specifications/desktopcouch/note";
    private static const string RECORD_TYPE_NOTE_VERSION = "1.0";
    private static const string RECORD_TYPE_TASK = 
        "http://www.freedesktop.org/wiki/Specifications/desktopcouch/task";
    private static const string RECORD_TYPE_TASK_VERSION = "1.0";
    private static const string RECORD_TYPE_PROJECT = 
        "http://launchpad.net/lucruri/project";
    private static const string RECORD_TYPE_PROJECT_VERSION = "1.0";
    private static const string RECORD_TYPE_CATEGORY = 
        "http://launchpad.net/lucruri/category";
    private static const string RECORD_TYPE_CATEGORY_VERSION = "1.0";
    /* signals */
    public signal void updated ();
    public signal void notFound ();
    public signal void accessDenied ();


	public delegate void voidFunc ();

	private CouchDB.Session _couchdb;
	private CouchDB.Session makeCouchDBSingleton() {
		    if (_couchdb == null) {
		        _couchdb = (new DesktopCouch.Session()) as CouchDB.Session;
		    }
		    return _couchdb;
    }
	public CouchDB.Session couchdb {
		get {
		    makeCouchDBSingleton();
		    return _couchdb;
		}
	}

    private CouchDB.Document _config;
	public CouchDB.Document config {
	    get {
	        if (_config == null) {
                _config = couchdb.get_document(
		            CouchDBBackend.DB_LUCRURI,
		            CouchDBBackend.DOCUMENT_CONFIG);
	        }
		    return _config;
        }
        set {
            _config = value;
        }
	}


	/* constructors */
	construct {
		log = Logger.getLogger("CouchDBBackend");
		_username = "";
		_password = "";
		_uri = "";
		tasks = new Gee.ArrayList<ITask>();
		notes = new Gee.ArrayList<INote>();
		projects = new Gee.ArrayList<IProject>();
		categories = new Gee.ArrayList<ICategory>();
		_couchdb = null;
		_config = null;
	}


    public CouchDBBackend() {
        //this.couchdb.enable_authentication(auth);
        //this.couchdb.authentication_failed.connect(() => {log.debug("CACACAC");});
	}

    /* methods */
    public void update(IBackend.voidFunc func){
    	updateUIFunc = func;
    	updateProjects();
    	updateCategories();
    	updateTasks();
    	updateNotes();
    	/* now that we are done, emit backend update signal */
    	updated();
    }
    
    public void updateProjects(){
        GLib.SList<CouchDB.DocumentInfo> couchdb_projects;
        couchdb_projects = couchdb.list_documents(DB_LUCRURI);
        foreach (
            CouchDB.DocumentInfo couchdb_project_info in couchdb_projects) {
            if (isNewItem (projects, couchdb_project_info.get_docid())) {
                CouchDB.Document couchdb_project = couchdb.get_document(
                    CouchDBBackend.DB_LUCRURI,
                    couchdb_project_info.get_docid());
                if (couchdb_project.get_string_field(
                    FIELD_RECORD_TYPE) == RECORD_TYPE_PROJECT) {

                    CouchDBProject project = new CouchDBProject.fromDocument(
                        this, couchdb_project);
                    projects.add(project);
                    project.updated.connect(updateUI);
                }

			}
        }
    }
 
    public void updateCategories(){
        GLib.SList<CouchDB.DocumentInfo> couchdb_categories;
        couchdb_categories = couchdb.list_documents(DB_LUCRURI);
        foreach (
            CouchDB.DocumentInfo couchdb_category_info in couchdb_categories) {
            if (isNewItem (categories, couchdb_category_info.get_docid())) {
                CouchDB.Document couchdb_category = couchdb.get_document(
                    CouchDBBackend.DB_LUCRURI,
                    couchdb_category_info.get_docid());
                if (couchdb_category.get_string_field(
                    FIELD_RECORD_TYPE) == RECORD_TYPE_CATEGORY) {

                    CouchDBCategory category = new CouchDBCategory.fromDocument(
                        this, couchdb_category);
                    categories.add(category);
                    category.updated.connect(updateUI);
                }

			}
        }
    }


    public void updateTasks(){
        GLib.SList<CouchDB.DocumentInfo> couchdb_tasks;
        couchdb_tasks = couchdb.list_documents(DB_TASKS);
        foreach (
            CouchDB.DocumentInfo couchdb_task_info in couchdb_tasks) {
            if (isNewItem (tasks, couchdb_task_info.get_docid())) {
                CouchDB.Document couchdb_task = couchdb.get_document(
                    CouchDBBackend.DB_TASKS,
                    couchdb_task_info.get_docid());
                CouchDBTask task = new CouchDBTask.fromDocument(
                    this, couchdb_task);
                tasks.add(task);
			    task.updated.connect(updateUI);
			}
        }
    }

    public void updateNotes(){
        GLib.SList<CouchDB.DocumentInfo> couchdb_notes;
        couchdb_notes = couchdb.list_documents(DB_NOTES);
        foreach (
            CouchDB.DocumentInfo couchdb_note_info in couchdb_notes) {
            if (isNewItem (notes, couchdb_note_info.get_docid())) {

                CouchDB.Document couchdb_note = couchdb.get_document(
                    CouchDBBackend.DB_NOTES,
                    couchdb_note_info.get_docid());
                CouchDBNote note = new CouchDBNote.fromDocument(
                    this, couchdb_note);
                notes.add(note);
                note.updated.connect(updateUI);
			}
        }
    }

    public void updateConfig (string key, string val) {
        if (this.config == null){
            this.config = new CouchDB.Document(this.couchdb);
        }
        this.config.set_string_field(key, val);
        this.config.set_id(CouchDBBackend.DOCUMENT_CONFIG);
        this.config.put(CouchDBBackend.DB_LUCRURI);
	}

    public string getConfig (string key) {
        if (this.config != null) {
		    return this.config.get_string_field(key);
		} else {
		    return "";
		}
	}

    public ITask newTask () {
        CouchDBTask task = new CouchDBTask(this, couchdb);
		this.tasks.add(task);
		task.updated.connect(updateUI);	
		return task;
	}

    public INote newNote () {
        CouchDBNote note = new CouchDBNote(this, couchdb);
		this.notes.add(note);
		note.updated.connect(updateUI);	
		return note;
	}

    public IProject newProject (bool add) {
        CouchDBProject project = new CouchDBProject(this, couchdb);
        if (add) {
		    this.projects.add(project);
		    project.updated.connect(updateUI);	
		}
		return project;
	}

    public ICategory newCategory (bool add) {
        CouchDBCategory category = new CouchDBCategory(this, couchdb);
        if (add) {
		    this.categories.add(category);
		    category.updated.connect(updateUI);
        }
		return category;
	}

    public static void getAnnotations(
            CouchDB.Document document,
            out CouchDB.StructField application_annotations,
            out CouchDB.StructField lucruri_annotations) {

        application_annotations = document.get_application_annotations();
        if (application_annotations == null) {
            application_annotations = new CouchDB.StructField.from_string(
                STRING_ANNOTATIONS);
            document.set_application_annotations(application_annotations);
        }

        lucruri_annotations = application_annotations.get_struct_field(
            FIELD_LUCRURI);
        if (lucruri_annotations == null) {
            lucruri_annotations = new CouchDB.StructField();
            application_annotations.set_struct_field(
                FIELD_LUCRURI,
                lucruri_annotations);
        }
    }

    public static void putDocument(
            IItem item,
            CouchDB.Document document,
            string database) {
	    if (document.put(database)){
	        if (item.id == null){
	            item.id = document.get_id();
	            item.created(true);
	        } else {
	            item.updated(true);
	       }
	    } else {
	        if (item.id == null){
	            item.created(false);
	        } else {
	            item.updated(false);
	        }
	    }
	}

    public static void fillItemFieldsFromDocument(
            IItem item,
            CouchDB.Document document){
        item.id = document.get_id();

        if (document.has_field(FIELD_NAME)) {
            item.name = document.get_string_field(
                FIELD_NAME);
        }

        if (document.has_field(FIELD_TEXT)) {
            item.text = document.get_string_field(
                FIELD_TEXT);
        }

        if (document.has_field(FIELD_CREATED_AT)) {
            item.createdAt = Utils.DateTime.parseTime(
                document.get_string_field(
                    FIELD_CREATED_AT),
                "%Y-%m-%dT%T");
        }

        if (document.has_field(FIELD_UPDATED_AT)) {
            item.updatedAt = Utils.DateTime.parseTime(
                document.get_string_field(
                    FIELD_UPDATED_AT),
                "%Y-%m-%dT%T");
        }
    }

    public static void fillDocumentFieldsFromItem(
            CouchDB.Document document,
            IItem item){

        document.set_string_field(
            CouchDBBackend.FIELD_NAME, item.name);
        document.set_string_field(
            CouchDBBackend.FIELD_TEXT, item.text);
        document.set_string_field(
            CouchDBBackend.FIELD_UPDATED_AT,
		    item.updatedAt.format("%Y-%m-%dT%H:%M:%S"));
        document.set_string_field(
            CouchDBBackend.FIELD_CREATED_AT,
		    item.createdAt.format("%Y-%m-%dT%H:%M:%S"));
    }

    /***************************************************************************
     * Private stuff
     */

	private void updateUI(bool status) {
		updateUIFunc();
	}

	private bool isNewItem(Gee.ArrayList<IItem>items, string docid) {
		foreach (IItem item in items) {
			if (item.id == docid) return false;
		}
		return true;
	}
}
