public class CouchDBCategory: GLib.Object, IItem, ICategory {
    /* properties */
    public string id { get; set; }

    public int width {get; set;}
    public int height {get; set;}

    public GLib.Time createdAt { get; set; }    
    public GLib.Time updatedAt { get; set; }
    public bool hide { get; set; }
    public int position { get; set; }
    public string name { set; get; }
    public string text { get; set; }
    public bool isDeleted {get; set;}

    public IBackend backend  { get; set;}
    public int instanceOf { get {return Utils.Constant.itemType.CATEGORY;}}

    /*signals*/
    public signal void created ( bool status );
    public signal void deleted ( bool status );
    public signal void updated ( bool status );

	private Logger log;
    private CouchDB.Document document;

    construct {
        id = null;
        name = _("Untitled Category");
        hide = false;
		text = "";
        createdAt = Utils.DateTime.newTimeNow();
		updatedAt = Utils.DateTime.newTimeNow();
		log = Logger.getLogger("SQLiteCategory");
    }

    public CouchDBCategory (IBackend aBackend, CouchDB.Session couchdb){
        this.backend = aBackend;
        this.document = new CouchDB.Document(couchdb);
        this.document.set_string_field(
            CouchDBBackend.FIELD_RECORD_TYPE,
		    CouchDBBackend.RECORD_TYPE_CATEGORY);
        this.document.set_string_field(
            CouchDBBackend.FIELD_RECORD_TYPE_VERSION,
		    CouchDBBackend.RECORD_TYPE_CATEGORY_VERSION);
	}

    public CouchDBCategory.fromDocument (IBackend aBackend, CouchDB.Document couchdb_category){
        this.backend = aBackend;
        this.document = couchdb_category;
        CouchDBBackend.fillItemFieldsFromDocument(this, couchdb_category);
        this.position = document.get_int_field(
            CouchDBBackend.FIELD_POSITION);
        this.hide = document.get_boolean_field(
            CouchDBBackend.FIELD_HIDE);
	}

	/* methods */

	public void push(){
        CouchDBBackend.fillDocumentFieldsFromItem(
            this.document,
            this);

        this.document.set_boolean_field(
            CouchDBBackend.FIELD_HIDE, this.hide);
        this.document.set_int_field(
            CouchDBBackend.FIELD_POSITION, this.position);

        CouchDBBackend.putDocument(
            this,
            this.document,
            CouchDBBackend.DB_LUCRURI);
	}

    public void delete (){
        if (this.document.delete()) {
		    this.isDeleted = true;
		    this.updated(true);
		}    
	}

    public void update (){
        this.updatedAt = Utils.DateTime.newTimeNow();
        this.push();
	}


/******************************************************************************
 * PRIVATE
 */


}
