public abstract class CouchDBItem : GLib.Object , IItem {
    /* properties */

    public string id {get; set;}
    public string name {get; set;}
    public string text {get; set;}
    public GLib.Time updatedAt {get; set;}
    public GLib.Time createdAt {get; set;}

    public int width {get; set;}
    public int height {get; set;}
    public int x {get; set;}
    public int y {get; set;}
    public bool isDeleted {get; set;}

    public IBackend backend  {get; set;}
    public abstract int instanceOf { get; }

	/* signals */
    public signal void created ( bool status );
    public signal void deleted ( bool status );
    public signal void updated ( bool status );

    protected CouchDB.Document document;

    construct {
        /*stored in db*/
        this.id = null;
        this.text = "";
        this.createdAt = Utils.DateTime.newTimeNow();
        this.updatedAt = Utils.DateTime.newTimeNow();

        /*temporary stored in memory*/
		this.isDeleted = false;
        this.height = 350;
        this.width = 450;
        this.x = 0;
        this.y = 0;
		this.backend = null;
        this.document = null;
    }

    public CouchDBItem (IBackend aBackend, CouchDB.Session couchdb){
        this.backend = aBackend;
        this.document = new CouchDB.Document(couchdb);
	}

    public CouchDBItem.fromDocument (IBackend aBackend, CouchDB.Document aDocument){
        this.backend = aBackend;
        this.document = aDocument;
        CouchDBBackend.fillItemFieldsFromDocument(this, aDocument);
	}

	public abstract void push();

    public void delete (){
        if (this.document.delete()) {
		    this.isDeleted = true;
		    this.updated(true);
		}    
	}

    public void update (){
        this.updatedAt = Utils.DateTime.newTimeNow();
        this.push();
	}

}
