public class CouchDBNote : GLib.Object , IItem, INote {
    /* properties */
    public string id {get; set;}
    public string name {get; set;}
    public string text {get; set;}
    public GLib.Time updatedAt {get; set;}
    public GLib.Time createdAt {get; set;}

    public int width {get; set;}
    public int height {get; set;}
    public int x {get; set;}
    public int y {get; set;}
    public bool isDeleted {get; set;}

    public IBackend backend  {get; set;}

    public string projectID {get; set;}
    public bool sticky {get; set;}

    public int instanceOf {get {return Utils.Constant.itemType.NOTE;}}
	private Logger log;

	/* signals */
    public signal void created ( bool status );
    public signal void deleted ( bool status );
    public signal void updated ( bool status );

    private CouchDB.Document document;


    construct {
        /*stored in db*/
        this.id = null;
        this.name = "Untitled Note";
        this.text = "";
		this.projectID = "0";
        this.createdAt = Utils.DateTime.newTimeNow();
        this.updatedAt = Utils.DateTime.newTimeNow();
		this.sticky = false;

        /*temporary stored in memory*/
		this.isDeleted = false;
        this.height = 350;
        this.width = 450;
        this.x = 0;
        this.y = 0;
		this.backend = null;
        this.document = null;
		log = Logger.getLogger("CouchDBNote");
    }

    public CouchDBNote (IBackend aBackend, CouchDB.Session couchdb){
        this.backend = aBackend;
        this.document = new CouchDB.Document(couchdb);
        this.document.set_string_field(
            CouchDBBackend.FIELD_RECORD_TYPE,
		    CouchDBBackend.RECORD_TYPE_NOTE);
        this.document.set_string_field(
            CouchDBBackend.FIELD_RECORD_TYPE_VERSION,
		    CouchDBBackend.RECORD_TYPE_NOTE_VERSION);
       
	}

    public CouchDBNote.fromDocument (IBackend aBackend, CouchDB.Document couchdb_note){
        this.backend = aBackend;
        this.document = couchdb_note;
        CouchDBBackend.fillItemFieldsFromDocument(this, couchdb_note);

        CouchDB.StructField application_annotations;
        CouchDB.StructField lucruri_annotations;
        CouchDBBackend.getAnnotations(
            couchdb_note,
            out application_annotations,
            out lucruri_annotations);
        this.sticky = lucruri_annotations.get_boolean_field(
            CouchDBBackend.FIELD_STICKY);
        this.projectID = lucruri_annotations.get_string_field(
            CouchDBBackend.FIELD_PROJECT_ID);
        if (this.projectID == null) {
            this.projectID = "";
        }

        this.width = lucruri_annotations.get_int_field(
            CouchDBBackend.FIELD_WIDTH);
        this.height = lucruri_annotations.get_int_field(
            CouchDBBackend.FIELD_HEIGHT);

	}

	public void push(){
        CouchDBBackend.fillDocumentFieldsFromItem(
            this.document,
            this);

        CouchDB.StructField application_annotations;
        CouchDB.StructField lucruri_annotations;
        CouchDBBackend.getAnnotations(
            this.document,
            out application_annotations,
            out lucruri_annotations);

        lucruri_annotations.set_string_field(
            CouchDBBackend.FIELD_PROJECT_ID, this.projectID);
        lucruri_annotations.set_boolean_field(
            CouchDBBackend.FIELD_STICKY, this.sticky);

        lucruri_annotations.set_int_field(
            CouchDBBackend.FIELD_WIDTH, this.width);
        lucruri_annotations.set_int_field(
            CouchDBBackend.FIELD_HEIGHT, this.height);

        CouchDBBackend.putDocument(
            this,
            this.document,
            CouchDBBackend.DB_NOTES);
	}

    public void delete (){
        if (this.document.delete()) {
		    this.isDeleted = true;
		    this.updated(true);
		}    
	}

    public void update (){
        this.updatedAt = Utils.DateTime.newTimeNow();
        this.push();
	}

/******************************************************************************
 * PRIVATE
 */



}
