public class CouchDBProject: GLib.Object, IItem, IProject {
    /* properties */
    public string id { get; set; }

    public int width {get; set;}
    public int height {get; set;}

    public GLib.Time completedAt { get; set; }
    public GLib.Time createdAt { get; set; }    
    public GLib.Time updatedAt { get; set; }
    public bool active { get; set; }
    public int position { get; set; }
    public string name { set; get; }
    public string text { set; get; }

    public bool isDeleted {get; set;}
    public IBackend backend  { get; set;}
    public int instanceOf { get {return Utils.Constant.itemType.PROJECT;}}

    /*signals*/
    public signal void created ( bool status );
    public signal void deleted ( bool status );
    public signal void updated ( bool status );

	private Logger log;
    private CouchDB.Document document;

    construct {
        id = null;
        name = _("Untitled Project");
        text = "Project description";
		position = 1000;
        updatedAt = Utils.DateTime.newTimeNow();       
        createdAt = Utils.DateTime.newTimeNow();
        completedAt = GLib.Time();
		log = Logger.getLogger("CouchDBProject");
    }
    
    public CouchDBProject (IBackend aBackend, CouchDB.Session couchdb){
        this.backend = aBackend;
        this.document = new CouchDB.Document(couchdb);
        this.document.set_string_field(
            CouchDBBackend.FIELD_RECORD_TYPE,
		    CouchDBBackend.RECORD_TYPE_PROJECT);
        this.document.set_string_field(
            CouchDBBackend.FIELD_RECORD_TYPE_VERSION,
		    CouchDBBackend.RECORD_TYPE_PROJECT_VERSION);
       
	}

    public CouchDBProject.fromDocument (IBackend aBackend, CouchDB.Document couchdb_project){
        this.backend = aBackend;
        this.document = couchdb_project;
        CouchDBBackend.fillItemFieldsFromDocument(this, couchdb_project);

        this.position = document.get_int_field(
            CouchDBBackend.FIELD_POSITION);
        
	}

	/* methods */

	public void push(){
        CouchDBBackend.fillDocumentFieldsFromItem(
            this.document,
            this);

        this.document.set_int_field(
            CouchDBBackend.FIELD_POSITION, this.position);

        CouchDBBackend.putDocument(
            this,
            this.document,
            CouchDBBackend.DB_LUCRURI);
	}

    public void delete (){
        if (this.document.delete()) {
		    this.isDeleted = true;
		    this.updated(true);
		}    
	}

    public void update (){
        this.updatedAt = Utils.DateTime.newTimeNow();
        this.push();
	}

/******************************************************************************
 * PRIVATE
 */

}
