public class CouchDBTask : GLib.Object, IItem, ITask {

    public string id {get; set;}
    public string name {get; set;}
    public string text {get; set;}
    public GLib.Time updatedAt {get; set;}
    public GLib.Time createdAt {get; set;}

    public int width {get; set;}
    public int height {get; set;}
    public int x {get; set;}
    public int y {get; set;}
    public bool isDeleted {get; set;}

    public IBackend backend  {get; set;}

    public bool done  { get; set; }
    public int priority  { get; set; }
    public string categoryID  { get; set; }
    public string projectID  { get; set; }

    public GLib.Time completedAt { get; set; }
    public GLib.Time dueAt { get; set; }
    public GLib.Time showFrom { get; set; }

    public int instanceOf { get {return Utils.Constant.itemType.TASK;}}
	private Logger log;

	/* signals */
    public signal void created ( bool status );
    public signal void deleted ( bool status );
    public signal void updated ( bool status );

    private CouchDB.Document document;


    construct {
        /*stored in db*/
        id = null;
        done = false;
        name = _("Task Name");
        priority = 0;
        text = "";
        projectID = "0";
        categoryID = "0";
        updatedAt = Utils.DateTime.newTimeNow();
        completedAt = GLib.Time();
        showFrom = GLib.Time();
        dueAt = GLib.Time();
        createdAt = Utils.DateTime.newTimeNow();

        /*temporary stored in memory*/
		log = Logger.getLogger("CouchDBTask");
    }

    public CouchDBTask (IBackend aBackend, CouchDB.Session couchdb) {
		this.backend = aBackend;
		this.document = new CouchDB.Document(couchdb);
        this.document.set_string_field(
            CouchDBBackend.FIELD_RECORD_TYPE,
		    CouchDBBackend.RECORD_TYPE_TASK);
        this.document.set_string_field(
            CouchDBBackend.FIELD_RECORD_TYPE_VERSION,
		    CouchDBBackend.RECORD_TYPE_TASK_VERSION);

	}

    public CouchDBTask.fromDocument (IBackend aBackend, CouchDB.Document couchdb_task){
        this.backend = aBackend;
        this.document = couchdb_task;
        CouchDBBackend.fillItemFieldsFromDocument(this, couchdb_task);

        this.done = this.document.get_boolean_field(
            CouchDBBackend.FIELD_DONE);

        this.dueAt = Utils.DateTime.parseTime(
            this.document.get_string_field(
                CouchDBBackend.FIELD_DUE_AT),
            "%Y-%m-%dT%T");

        this.showFrom = Utils.DateTime.parseTime(
            this.document.get_string_field(
                CouchDBBackend.FIELD_SHOW_FROM),
            "%Y-%m-%dT%T");

        this.completedAt = Utils.DateTime.parseTime(
            this.document.get_string_field(
                CouchDBBackend.FIELD_COMPLETED_AT),
            "%Y-%m-%dT%T");

        CouchDB.StructField application_annotations;
        CouchDB.StructField lucruri_annotations;
        CouchDBBackend.getAnnotations(
            couchdb_task,
            out application_annotations,
            out lucruri_annotations);

        this.categoryID = lucruri_annotations.get_string_field(
            CouchDBBackend.FIELD_CATEGORY_ID);

        this.projectID = lucruri_annotations.get_string_field(
            CouchDBBackend.FIELD_PROJECT_ID);

        this.width = lucruri_annotations.get_int_field(
            CouchDBBackend.FIELD_WIDTH);
        this.height = lucruri_annotations.get_int_field(
            CouchDBBackend.FIELD_HEIGHT);

    }

	public void push(){
        CouchDBBackend.fillDocumentFieldsFromItem(
            this.document,
            this);

        this.document.set_boolean_field(
            CouchDBBackend.FIELD_DONE, this.done);

        this.document.set_string_field(
            CouchDBBackend.FIELD_DUE_AT,
            this.dueAt.format("%Y-%m-%dT%H:%M:%S"));

        this.document.set_string_field(
            CouchDBBackend.FIELD_COMPLETED_AT,
            this.completedAt.format("%Y-%m-%dT%H:%M:%S"));

        this.document.set_string_field(
            CouchDBBackend.FIELD_SHOW_FROM,
            this.showFrom.format("%Y-%m-%dT%H:%M:%S"));

        CouchDB.StructField application_annotations;
        CouchDB.StructField lucruri_annotations;
        CouchDBBackend.getAnnotations(
            this.document,
            out application_annotations,
            out lucruri_annotations);

        lucruri_annotations.set_string_field(
            CouchDBBackend.FIELD_CATEGORY_ID, this.categoryID);
        lucruri_annotations.set_string_field(
            CouchDBBackend.FIELD_PROJECT_ID, this.projectID);

        lucruri_annotations.set_int_field(
            CouchDBBackend.FIELD_WIDTH, this.width);
        lucruri_annotations.set_int_field(
            CouchDBBackend.FIELD_HEIGHT, this.height);

        CouchDBBackend.putDocument(
            this,
            this.document,
            CouchDBBackend.DB_TASKS);
	}

    public void delete (){
        if (this.document.delete()) {
		    this.isDeleted = true;
		    this.updated(true);
		}    
	}

    public void update (){
        this.updatedAt = Utils.DateTime.newTimeNow();
        this.push();
	}  
/******************************************************************************
 * PRIVATE
 */

}
