using Gtk;

public class HomeView : Gtk.VPaned, IView {

	private Logger log;
	private IBackend backend;
    private Gtk.EventBox tasksBox;
    private Gtk.EventBox notesBox;

    private Gtk.Widget editArea;
	private NotesTreeView notesView;
	private Gtk.TreeModelFilter notesFilter;

	private List<TasksGroup> groups;
    private TasksTreeView tasksView;
	private Gtk.TreeModelFilter tasksFilter;

	private ScrolledWindow tasksOverview;
	private ScrolledWindow tasksSearch;
	private string searchString;

	construct {
		log = Logger.getLogger("HomeView");
		searchString = "";

        Adjustment hScrollAdjustment;
        Adjustment vScrollAdjustment;
        hScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        vScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        tasksOverview = new ScrolledWindow (hScrollAdjustment, vScrollAdjustment);
        tasksOverview.hscrollbar_policy = PolicyType.NEVER;
        tasksOverview.vscrollbar_policy = PolicyType.AUTOMATIC;

        hScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        vScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        tasksSearch = new ScrolledWindow (hScrollAdjustment, vScrollAdjustment);
        tasksSearch.hscrollbar_policy = PolicyType.NEVER;
        tasksSearch.vscrollbar_policy = PolicyType.AUTOMATIC;


	}

    public HomeView (IBackend aBackend, Gtk.Widget anEditArea)
    {
		backend = aBackend;
		editArea = anEditArea;
		
        notesBox = new Gtk.EventBox();
        tasksBox = new Gtk.EventBox();

		createNotesOverview();
		createTasksOverview();

		position = 200;

		pack1 (notesBox, false, false);
		pack2 (tasksBox, false, false);

    }

	public void search (string text) {
		this.searchString = text;
		//TODO add timeout for search
		this.notesFilter.refilter();
		if ((searchString == "") || (searchString == Utils.Constant.searchAll)){
			/* show tasks overview */
			if (tasksBox.get_child() != tasksOverview) {
				tasksBox.remove(tasksSearch);
				tasksBox.add(tasksOverview);
				tasksBox.show_all();
			}
		} else {
			/* search task */
			if (tasksBox.get_child() != tasksSearch) {
				tasksBox.remove(tasksOverview);
				tasksSearch.show_all();
				tasksBox.add(tasksSearch);
			}
			this.tasksFilter.refilter();
		}


	}

	private void createNotesOverview () {
		notesView = new NotesTreeView();
		
		Adjustment hScrollAdjustment;
        Adjustment vScrollAdjustment;
		ScrolledWindow scrolledWindow;

        hScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        vScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        scrolledWindow = new ScrolledWindow (hScrollAdjustment, vScrollAdjustment);
        scrolledWindow.hscrollbar_policy = PolicyType.NEVER;
        scrolledWindow.vscrollbar_policy = PolicyType.AUTOMATIC;
        scrolledWindow.add_with_viewport(notesView);
		notesBox.add(scrolledWindow);
	}

	private void createTasksOverview () {

        Gtk.VBox box = new Gtk.VBox(false,0);
		box.border_width = 5;        
		groups = new List<TasksGroup>();
        TasksGroup categoriesGroup;
        foreach (ICategory category in this.backend.categories) {
             if (! category.hide) {
                categoriesGroup = new TasksGroup(category.name, TasksGroup.groupType.HOME, category.id, backend, this.editArea);
				groups.append(categoriesGroup);
                box.pack_start(categoriesGroup,false,false,0);
            }
         }

        //add end spacer
        Label endSpacer = new Label("");
        box.pack_start(endSpacer,true,true,0);

		
		Gdk.Color backgroundColor = getBackgroundColor ();
		Gtk.EventBox tasksEvBox = new Gtk.EventBox();
		tasksEvBox.modify_bg (StateType.NORMAL, backgroundColor);
		tasksEvBox.modify_base (StateType.NORMAL, backgroundColor);
		tasksEvBox.add(box);
	
        tasksOverview.add_with_viewport(tasksEvBox);

		/*just create the tasksView and populate it */
        Gtk.TreeIter ti;
        Gtk.ListStore treeModel = new Gtk.ListStore (1, typeof(ITask));
        foreach (ITask task in this.backend.tasks) {
            treeModel.append (out ti);
            treeModel.set (ti, 0, task);
        }
 		tasksFilter = new Gtk.TreeModelFilter (treeModel, null);
		tasksFilter.set_visible_func(filterTasks);

		tasksView = new TasksTreeView (editArea, false);
		tasksView.set_model(this.tasksFilter);
		tasksSearch.add(tasksView);
		/*fist show the overview */
		tasksBox.add(tasksOverview);
	}


	public void updateAll() {
		Gtk.Widget? old = tasksBox.get_child();
		if (old != null) tasksBox.remove(old);				
		createTasksOverview();
		update();
	}

	public void update() {
		updateNotesModel();
		this.notesView.set_model ( this.notesFilter );
		this.notesFilter.refilter();

//		foreach (ITask task in backend.tasks) {
//			//FIXME memory leak
//			task.updated += () => { update();};
//		}

		foreach (TasksGroup group in groups) {
			group.update();
		}

	}

	private void updateNotesModel() {
        Gtk.TreeIter ti;
        Gtk.ListStore treeModel = new Gtk.ListStore (1, typeof(INote));

        /*
         unpack the task list and store it in a tree model
        */
		Utils.Backend.sortNotesByUpdatedAtDesc( this.backend.notes );
        foreach (INote note in this.backend.notes) {
            treeModel.append (out ti);
            treeModel.set (ti, 0, note);
			//FIXME memory leak
			//note.updated += () => { update();};
        }

 		notesFilter = new Gtk.TreeModelFilter (treeModel, null);
		notesFilter.set_visible_func( filterNotes );
//        foreach (INote note in this.backend.notes) {
//			//FIXME memoryleak
//			note.updated += () => { this.update();};
//        }
	}


    private bool filterNotes (Gtk.TreeModel model, Gtk.TreeIter iter) {
 	    INote note;
	    model.get (iter, 0, out note);
		if ((note == null) || (note.isDeleted)) return false;
		if ((searchString == "")||(searchString == Utils.Constant.searchAll)) {
		    int age = Utils.DateTime.newDateFromTime(
		        note.updatedAt).days_between(Utils.DateTime.newDateNow());
			if (note.sticky || age < 8){
	            return true;
			} else {
				return false;
			}
        } else if ((note.name.casefold().str(searchString.casefold()) != null) || 
				  		(note.text.casefold().str(searchString.casefold()) != null)) {
			return true;
		} else {
			return false;
		}
		
    }


    private bool filterTasks (Gtk.TreeModel model, Gtk.TreeIter iter) {
 	    ITask task;
	    model.get (iter, 0, out task);
		if ((task == null) || (task.isDeleted)) return false;
		if ((searchString == "") || 
			(searchString == Utils.Constant.searchAll) || 
			(task.name.casefold().str(searchString.casefold()) != null) || 
			(task.text.casefold().str(searchString.casefold()) != null)) {
			return true;
		} else {
			return false;
		}
		
    }


    private Gdk.Color getBackgroundColor () {
		Gtk.Style style = Gtk.rc_get_style (this);
		Gdk.Color color = style.base [(int)Gtk.StateType.NORMAL];
		return color;
	}
}
