public interface IBackend : GLib.Object {
    /* properties */
    public abstract Gee.ArrayList<ITask> tasks { get; set; }
    public abstract Gee.ArrayList<ICategory> categories { get; set;}
    public abstract Gee.ArrayList<IProject> projects { get; set; } 
    public abstract Gee.ArrayList<INote> notes { get; set; } 

    public abstract string name { get; }
    public abstract bool isConfigured { get; }
    
    public abstract string username { set; }
    public abstract string password { set; }
    public abstract string uri { set; }
//FIXME include IBackend in this interface
//    public abstract IBackend.voidFunc updateUIFunc { get; set; }

    /*signals*/
    public abstract signal void updated ();

    public abstract signal void notFound ();
    public abstract signal void accessDenied ();

    /*methods*/
	/* the backend will just creaty empty objects
     * bool values specifies if the new object should be added to the internal data ( but not commited)
     * UI should populate them and
     * push (commit) the changes back to backend
     */
    public abstract ITask newTask ();
    public abstract INote newNote ();
	public abstract IProject newProject (bool add);
	public abstract ICategory newCategory (bool add);

	/*
	 * Request backend to udate/refresh it's internal data 
     */
    public abstract void updateTasks ();
    public abstract void updateProjects ();
    public abstract void updateCategories ();
    public abstract void updateNotes ();
    public abstract void update(IBackend.voidFunc func);

    public abstract void updateConfig (string key, string val);
    public abstract string getConfig (string key);

    public delegate void voidFunc ();

}
