using GLib;
using Gtk;
using Gdk;
using Panel;

public class LucruriGNOMEApplet : Panel.Applet {
    public static Configuration config;
    public static IBackend backend;
    public static MainWindow mainWindow;
    private Gtk.Image icon;

    private void create () {
        //first create the icon
        icon = new Gtk.Image.from_pixbuf(Utils.UI.getPixbuf("/icons/22x22/emotes/face-monkey.png"));
        add (icon);

        initBackend();
        initRightClickMenu();

		//FIXME
        //this.change_background += this.on_change_background;
		this.button_press_event += this.on_button_press_release;

		mainWindow = new MainWindow ( 
							(this.parent as LucruriGNOMEApplet).backend ,
							(this.parent as LucruriGNOMEApplet).config 
							);
		mainWindow.update();

		mainWindow.delete_event += this.mainWindow.hide_on_delete; 
		mainWindow.destroy += () => {};

		this.show_all ();
    }
        
    private bool on_button_press_release (LucruriGNOMEApplet sender, Gdk.EventButton event) {

        if (event.type == Gdk.EventType.BUTTON_PRESS && event.button == 1) {
            //if ((event.state & Gdk.ModifierType.SHIFT_MASK) != 0) {
			AppletMenu appletMenu = new AppletMenu(backend, icon);
	        appletMenu.show_all();
    	    appletMenu.popup (null, null, Utils.UI.getMenuPosition, 1, Gtk.get_current_event_time ());
            return true;
        }

        return false;
    }

    private void on_change_background (LucruriGNOMEApplet applet, Panel.AppletBackgroundType type, ref Gdk.Color color, ref Gdk.Pixmap pixmap) {
        applet.set_style (null);

        var rc_style = new Gtk.RcStyle ();
        applet.modify_style (rc_style);

        switch (type) {
        case Panel.AppletBackgroundType.COLOR_BACKGROUND:
            applet.modify_bg (Gtk.StateType.NORMAL, color);
            break;
        case Panel.AppletBackgroundType.PIXMAP_BACKGROUND:
            applet.style.bg_pixmap[0] = pixmap;
            set_style (style);
            break;
        }
    }

    private bool requestBackendUpdate () {
		this.backend.update(() => {});
        return true;
    }

    private void initBackend() {
	    // FIXME show config if not configured


		this.backend.updated.connect( initialBackendUpdate );
        //make an initial request
        requestBackendUpdate();
	
        //refresh at every 5 minutes 
		//TODO add config value
        GLib.Timeout.add_seconds(5 * 60 , requestBackendUpdate);        
    }

	public void initialBackendUpdate () {
		config.initOpenedWindows();
		this.backend.updated.disconnect( initialBackendUpdate );
	}

    private void initRightClickMenu () {
	
        string menu_definition = 
            "<popup name=\"button3\">" +
            "<menuitem debuname=\"About\" verb=\"About\" label=\"%s\" pixtype=\"stock\" pixname=\"gnome-stock-about\"/>" +
            "</popup>";

        BonoboUI.Verb[] verbs = new BonoboUI.Verb[2];

/*        verbs[0].cname = "Configuration";
        verbs[0].cb = on_preferences_clicked;
		verbs[0].user_data = this;
*/
        verbs[0].cname = "About";
        verbs[0].cb = on_about_clicked;
		verbs[0].user_data = this;

		verbs[1].cname = null;
		verbs[1].cb = null;
		verbs[1].user_data = null;

        setup_menu (menu_definition.printf( _("About")), verbs, null);
    }

    private static void on_about_clicked (BonoboUI.Component component,
                                          void* user_data, string cname) {
        AboutDialog dialog = new AboutDialog ();
        dialog.show ();
    }

    private static void on_preferences_clicked (BonoboUI.Component component,
                                          void* user_data, string cname) {

        LucruriGNOMEApplet applet = (user_data as LucruriGNOMEApplet);
        PreferencesDialog preferencesDialog = new PreferencesDialog( applet.config );
        preferencesDialog.response += ( sender ) => {
    	    applet.backend.uri = applet.config.tracks_uri;
        };
        preferencesDialog.run();
        preferencesDialog.destroy();
    }


/*******************************************************************************
 *
 */

    public static bool factory (Applet applet, string iid) {
        (applet as LucruriGNOMEApplet).create ();
        return true;
    }

    public static int main (string[] args) {

        // this is a workaround to make Vala include config.h before
        // gi18n-lib.h
        string gettext_package = Config.GETTEXT_PACKAGE;
        Intl.bindtextdomain(gettext_package, Config.PACKAGE_LOCALEDIR);
        Intl.bind_textdomain_codeset(Config.GETTEXT_PACKAGE, "UTF-8");
        Intl.textdomain(Config.GETTEXT_PACKAGE);

        Gnome.Program.init ("Lucruri_Applet", "0", Gnome.libgnomeui_module, args, "sm-connect", false);

        //set up backend and preferences
        backend = new CouchDBBackend();
        config = new Configuration(backend);
    
        int ret = Panel.Applet.factory_main ("OAFIID:Lucruri_Applet_Factory",
                                    typeof (LucruriGNOMEApplet), factory);
		config.sync();
		return ret;
    }
}
