using Gtk;
public class NoteMenuItem: Gtk.ImageMenuItem {

	public INote note { get; set;}

	private Gtk.Image pinImage;
	private Gtk.Label label;

	private Gdk.Pixbuf pinUp;
	private Gdk.Pixbuf pinDown;
	private Gdk.Pixbuf pinHover;	

	private Logger log;

	construct {
		pinUp = Utils.UI.getPixbuf("/icons/16x16/status/pin-up.png");
		pinDown = Utils.UI.getPixbuf("/icons/16x16/status/pin-down.png");
		pinHover = Utils.UI.getPixbuf("/icons/16x16/status/pin-active.png");
		log = Logger.getLogger("NoteMenuItem");
	}

	public NoteMenuItem (INote note) {
			this.note = note;

            this.image = new Gtk.Image.from_pixbuf(Utils.UI.getPixbuf("/icons/16x16/places/note.png"));		
			pinImage = new Gtk.Image.from_pixbuf((note.sticky) ? pinDown : pinUp);
			label = new Gtk.Label(Utils.UI.ellipsify(note.name,50));
			Gtk.Label spacer = new Gtk.Label("");
			Gtk.HBox box = new Gtk.HBox(false, 0);
			Gtk.Widget oldChild = this.child;
			remove(oldChild);
			add(box);
			
            Utils.UI.setTooltip(label, Utils.UI.ellipsify(this.note.text, 400));

			box.pack_start(oldChild, false, false, 0);
			box.pack_start(label, false, false, 0);
			box.pack_start(spacer, true, true, 0);
			box.pack_start(pinImage, false, false, 0);
			this.show_all();			
	}

	public override bool motion_notify_event (Gdk.EventMotion ev) {
		if (!this.note.sticky && pinImage != null) {
			if (ev.x >= pinImage.allocation.x && 
				ev.x < pinImage.allocation.x + pinImage.allocation.width) {
				if (pinImage.pixbuf != pinHover){
					pinImage.pixbuf = pinHover;
				}
			} else if ( pinImage.pixbuf != pinUp) {
				pinImage.pixbuf = pinUp;
			}
		}
		return true;
	}

	public override bool button_press_event (Gdk.EventButton ev) {
		if (pinImage != null) {
			if (ev.x >= pinImage.allocation.x && 
				ev.x < pinImage.allocation.x + pinImage.allocation.width) {
				if (note.sticky){
					note.sticky = false;
					pinImage.pixbuf = pinUp;
				} else {
					note.sticky = true;
					pinImage.pixbuf = pinDown;
				}
				note.update();
				//FIXME show menu after an action
				//(this.parent as AppletMenu).showMenu();
			} else {
				Utils.UI.showNoteWindow(note);
			}
		}
		return true;
	}

	public override bool leave_notify_event (Gdk.EventCrossing ev) {
		if (!this.note.sticky && pinImage != null) {
			pinImage.pixbuf = pinUp;
		}
		return true;
	}

}

