using Gtk;

public class NotesView : Gtk.HPaned, IView {

    private NotesTreeView notesView;
	private ProjectsTreeView projectsView;
	private Gtk.TreeModelFilter filter;
	private Gtk.TreeModelFilter projectsFilter;
	private IBackend backend;
	private Gtk.TreeIter selectedIter;


	private string searchString;
	private Logger log;

	construct {
		searchString = "";
		log = Logger.getLogger("NotesView");
	}

    public NotesView (IBackend aBackend)
    {
		backend = aBackend;

		projectsView = new ProjectsTreeView (false);
		projectsView.currentFilterRule = projectsView.ALL_NOTES;

		notesView = new NotesTreeView ();


        Adjustment hScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        Adjustment vScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        ScrolledWindow scrolledWindow = new ScrolledWindow (hScrollAdjustment, vScrollAdjustment);
        scrolledWindow.hscrollbar_policy = PolicyType.NEVER;
        scrolledWindow.vscrollbar_policy = PolicyType.AUTOMATIC;

        scrolledWindow.add(notesView);

		position = 150;
		pack1 (projectsView, false, false);
		pack2 (scrolledWindow, true, false);
    }

	public void update() {
			updateNotesModel();
			this.notesView.set_model ( this.filter );
			projectsView.targetModel = this.filter;
			this.filter.refilter();

			updateProjectsModel();
			this.projectsView.set_model ( this.projectsFilter );
			this.projectsFilter.refilter();
			Gtk.TreePath? path = this.projectsFilter.get_model().get_path (this.selectedIter);
			if (path != null) this.projectsView.set_cursor(path, null, false);


	}

	public void search(string text) {
		this.searchString = text;
		this.filter.refilter();
	}

/******************************************************************************
 * PRIVATE
 */





	private void updateNotesModel() {
        Gtk.TreeIter ti;
        Gtk.ListStore treeModel = new Gtk.ListStore (2, typeof(INote), typeof (string));

        /*
         unpack the task list and store it in a tree model
        */
		Utils.Backend.sortNotesByUpdatedAtDesc( this.backend.notes );
        foreach (INote note in this.backend.notes) {
            treeModel.append (out ti);
            treeModel.set (ti, 0, note, 1, note.name);
        }

 		filter = new Gtk.TreeModelFilter (treeModel, null);
		filter.set_visible_func( filterNotesTree );
	}

    private bool filterNotesTree (Gtk.TreeModel model, Gtk.TreeIter iter) {
 	    INote note;
	    model.get (iter, 0, out note);
		if ((note == null) || (note.isDeleted)) return false;

	    if (
			( 
				(projectsView.currentFilterRule == ProjectsTreeView.ALL_NOTES) || 
				(note.projectID == projectsView.currentFilterRule) ||
				((projectsView.currentFilterRule == ProjectsTreeView.PINNED_NOTES) && (note.sticky) )
			) &&
			( (searchString == "") || 
			  (note.name.casefold().str(searchString.casefold()) != null) || 
			  (note.text.casefold().str(searchString.casefold()) != null) )
			) {
            return true;
        } else {
           return false;
		}
    }


	private void updateProjectsModel() {

        Gtk.TreeIter ti;
        Gtk.ListStore treeModel = new Gtk.ListStore (1, typeof(IProject));

        /*
         unpack the task list and store it in a tree model
        */
		IProject meta;
		meta = backend.newProject(false);
		meta.name = "<b>" + _("All Notes") + "</b>";
		meta.id = ProjectsTreeView.ALL_NOTES;
        treeModel.append (out ti);
        treeModel.set (ti, 0, meta);
		selectedIter = ti;

		meta = backend.newProject(false);
		meta.name = "<b>" + _("Unfiled Notes") + "</b>";
		meta.id = projectsView.UNFILED_NOTES;
        treeModel.append (out ti);
        treeModel.set (ti, 0, meta);

		meta = backend.newProject(false);
		meta.name = "<b>" + _("Pinned Notes") + "</b>";
		meta.id = ProjectsTreeView.PINNED_NOTES;
        treeModel.append (out ti);
        treeModel.set (ti, 0, meta);

		Utils.Backend.sortProjectsByPositionAsc(this.backend.projects);
        foreach (IProject project in this.backend.projects) {
            	treeModel.append (out ti);
            	treeModel.set (ti, 0, project);
        }
		projectsFilter = new Gtk.TreeModelFilter (treeModel, null);
		projectsFilter.set_visible_func( filterProjectsTree );	
	}

    private bool filterProjectsTree (Gtk.TreeModel model, Gtk.TreeIter iter) {
 	    IProject project;
	    model.get (iter, 0, out project);
		if (project == null) return false;

	    if (!(project.isDeleted)
			) {
            return true;
        } else {
           return false;
		}	 
    }
}
