public class ProjectsTreeView : Gtk.TreeView {


	public static const string ALL_PROJECTS = "_all_projects";
	public static const string PINNED_NOTES = "_all_pinned";
	public static const string ALL_NOTES = "_all_notes";
	public static const string UNFILED_NOTES = "";

	public string currentFilterRule;
	public Gtk.TreeModelFilter targetModel;
	public bool editable;

	public Gtk.CellRendererText nameRenderer;

	construct {
		editable = true;
	}

	public ProjectsTreeView (bool edit) {

		rules_hint = false;
		headers_visible = true;
		hover_selection = false;

		if (edit) {
			editable = true;
			reorderable = true;
		} else {
			editable = false;
			reorderable = false;
		}


		get_selection().set_mode(Gtk.SelectionMode.BROWSE);

        Gtk.TreeViewColumn column;


		//
		// Project Name Column
		//
		column = new Gtk.TreeViewColumn ();
		// Title for Column
		column.title = _("Projects");
		column.sizing = Gtk.TreeViewColumnSizing.AUTOSIZE;
		column.expand = true;
		column.resizable = true;

		Gtk.CellRendererPixbuf editRenderer = new Gtk.CellRendererPixbuf ();
		column.pack_start (editRenderer, false);
		column.set_cell_data_func (
			editRenderer,
			(cell_layout, cell, model, iter) =>	{
				Gtk.CellRendererPixbuf crp = cell as Gtk.CellRendererPixbuf;
        		IProject project;
				model.get (iter, 0, out project);
				if (project.id == this.ALL_NOTES) {
					crp.pixbuf = Utils.UI.getPixbuf ("/icons/22x22/actions/filter-note-all.png" );
				} else if (project.id == this.UNFILED_NOTES) {
					crp.pixbuf = Utils.UI.getPixbuf ("/icons/22x22/actions/filter-note-unfiled.png" );
				} else if (project.id == this.PINNED_NOTES) {
					crp.pixbuf = Utils.UI.getPixbuf ("/icons/22x22/places/pinned.png" );
				} else {
        			crp.pixbuf = Utils.UI.getPixbuf ("/icons/22x22/actions/note-full.png" );
				}
    		}
			);


		
		nameRenderer = new Gtk.CellRendererText ();
		column.pack_start (nameRenderer, true);
		column.set_cell_data_func (
			nameRenderer,
			(cell_layout, cell, model, iter) =>	{
				Gtk.CellRendererText crt = cell as Gtk.CellRendererText;
				crt.ellipsize = Pango.EllipsizeMode.END;

        		IProject project;
				model.get (iter, 0, out project);
				crt.text = project.name;
				crt.markup = project.name;
    		}
			);
		nameRenderer.editable = true;

		
		append_column (column);
	}

	public IProject? getSelectedProject() {
		Gtk.TreePath path;
		Gtk.TreeViewColumn column;
		get_cursor (out path, out column);
		if (path == null) return null;

        Gtk.TreeIter iter;
        IProject project;
        if (!model.get_iter (out iter, path))  return null; // Do nothing
	    model.get (iter, 0, out project);	
        return project;
	}


	public override bool button_press_event (Gdk.EventButton eventButton) {
        Gtk.TreePath path;
		Gtk.TreeViewColumn column;

		if (targetModel == null)
			return base.button_press_event(eventButton);		

		int cell_x;
		int cell_y;
		get_path_at_pos (
			(int)eventButton.x, (int)eventButton.y, 
			out path, out column, out cell_x, out cell_y
		);
		if (path == null) return false; // Do nothing
        Gtk.TreeIter iter;
        IProject project;
        if (!model.get_iter (out iter, path))  return false; // Do nothing
	    model.get (iter, 0, out project);	
        if ( project == null ) return false;	 // if error .. do nothing
		currentFilterRule = project.id;
		targetModel.refilter();
		if ((project.id == ALL_PROJECTS)||
			(project.id == ALL_NOTES) ||
			(project.id == UNFILED_NOTES) ||
			(editable == false) 
			) { 
			set_cursor(path, null, false);
			return true;
		} else { 
			//set_cursor(path, null, true);		
			return base.button_press_event(eventButton);
		}
    }



}
