using Gtk;

public class RecoveryView : Gtk.HPaned {

    private TreeView itemsView;
	private TreeView itemsTypeView;
	private Gtk.TreeModelFilter filter;
	private IBackend backend;

	private string searchString;
	private Logger log;
	private string currentFilterRule;

	public static const string ALL = "-3";
	public static const string NOTES = "-2";
	public static const string TASKS = "-1";

	construct {
		searchString = "";
		log = Logger.getLogger("RecoverView");
	}

    public RecoveryView (IBackend aBackend)
    {
    	log.debug("Creating RecoverView");
		backend = aBackend;

		currentFilterRule = ALL;
		createIItemsTypeView ();
		createIItemsView ();

        Adjustment hScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        Adjustment vScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        ScrolledWindow scrolledWindow = new ScrolledWindow (hScrollAdjustment, vScrollAdjustment);
        scrolledWindow.hscrollbar_policy = PolicyType.NEVER;
        scrolledWindow.vscrollbar_policy = PolicyType.AUTOMATIC;

        scrolledWindow.add(itemsView);

		position = 150;
		pack1 (itemsTypeView, false, false);
		pack2 (scrolledWindow, true, false);

    }

	public void update() {
			updateIItemsModel();
			this.itemsView.set_model ( this.filter );
			this.filter.refilter();

	}

	public void filterSearch(string text) {
		this.searchString = text;
		//TODO add timeout for search
		this.filter.refilter();
	}

/******************************************************************************
 * PRIVATE
 */

	private void createIItemsTypeView () {
		itemsTypeView = new TreeView();
		itemsTypeView.rules_hint = false;
		itemsTypeView.headers_visible = true;
		itemsTypeView.hover_selection = false;
		itemsTypeView.reorderable = false;
		itemsTypeView.get_selection().set_mode(Gtk.SelectionMode.BROWSE);

        Gtk.TreeViewColumn column;


		//
		// Project Name Column
		//
		column = new Gtk.TreeViewColumn ();
		// Title for Column
		column.title = "Type Column";
		column.sizing = Gtk.TreeViewColumnSizing.AUTOSIZE;
		column.expand = true;
		column.resizable = true;

		Gtk.CellRendererPixbuf editRenderer = new Gtk.CellRendererPixbuf ();
		column.pack_start (editRenderer, false);
		column.set_cell_data_func (
			editRenderer,
			(cell_layout, cell, model, iter) =>	{
				Gtk.CellRendererPixbuf crp = cell as Gtk.CellRendererPixbuf;
        		IProject project;
				model.get (iter, 0, out project);
				if (project.id == this.NOTES) {
					crp.pixbuf = Utils.UI.getPixbuf ("/icons/22x22/actions/filter-note-all.png" );
				} else if (project.id == this.TASKS) {
					crp.pixbuf = Utils.UI.getPixbuf ("/icons/22x22/actions/note-full.png" );
				} else {
        			crp.pixbuf = Utils.UI.getPixbuf ("/icons/22x22/actions/filter-note-unfiled.png" );
				}
    		}
			);
		Gtk.CellRendererText nameRenderer = new Gtk.CellRendererText ();
		column.pack_start (nameRenderer, true);
		column.set_cell_data_func (
			nameRenderer,
			(cell_layout, cell, model, iter) =>	{
				Gtk.CellRendererText crt = cell as Gtk.CellRendererText;
				crt.ellipsize = Pango.EllipsizeMode.END;

        		IProject project;
				model.get (iter, 0, out project);
				crt.text = project.name;
				crt.markup = project.name;
    		}
			);
		nameRenderer.editable = false;
		itemsTypeView.append_column (column);

		itemsTypeView.button_press_event += (sender, eventButton) => {
		    Gtk.TreePath path;
			Gtk.TreeViewColumn aColumn;

			if (this.filter == null)
				return false;		

			int cell_x;
			int cell_y;
			sender.get_path_at_pos (
				(int)eventButton.x, (int)eventButton.y, 
				out path, out aColumn, out cell_x, out cell_y
			);
			if (path == null) return false; // Do nothing
		    Gtk.TreeIter iter;
		    GenericItem item;
		    if (!sender.model.get_iter (out iter, path))  return false; // Do nothing
			sender.model.get (iter, 0, out item);	
		    if ( item == null ) return false;	 // if error .. do nothing
			this.currentFilterRule = item.id;
			this.filter.refilter();
			sender.set_cursor(path, null, false);
			return true;
		};


        Gtk.TreeIter ti;
		Gtk.TreeIter firstIter;
        Gtk.ListStore treeModel = new Gtk.ListStore (1, typeof(IProject));

        /*
         unpack the task list and store it in a tree model
        */
		GenericItem meta;
		meta = new GenericItem();
		meta.name = "<b>" + _("All") + "</b>";
		meta.id = ALL;
        treeModel.append (out ti);
        treeModel.set (ti, 0, meta);
		firstIter = ti;


		meta = new GenericItem();
		meta.name = "<b>" + _("Only Notes") + "</b>";
		meta.id = NOTES;
        treeModel.append (out ti);
        treeModel.set (ti, 0, meta);

		meta = new GenericItem();
		meta.name = "<b>" + _("Only Tasks") + "</b>";
		meta.id = TASKS;
        treeModel.append (out ti);
        treeModel.set (ti, 0, meta);

		itemsTypeView.set_model( treeModel );	
		Gtk.TreePath path = treeModel.get_path (firstIter);
		itemsTypeView.set_cursor(path, null, false);
	}

	private void createIItemsView ()
	{
		itemsView = new Gtk.TreeView();
		itemsView.get_selection().set_mode(Gtk.SelectionMode.BROWSE);
       	itemsView.row_activated += ( obj, path, column) => {
            Gtk.TreeIter iter;
            IItem item;
            if (!itemsView.model.get_iter (out iter, path))  return; // Do nothing
		    itemsView.model.get (iter, 0, out item);	
            if ( item == null ) return;	 // if error .. do nothing
		
			if (item.instanceOf == Utils.Constant.itemType.NOTE) {
		        var noteWindow = new NoteWindow((INote)item);
				noteWindow.show_all();
			} else {
		        var taskWindow = new TaskWindow((ITask)item);
				taskWindow.show_all();
			}
        };

		updateIItemsModel();
 		itemsView.set_model ( filter );

		itemsView.rules_hint = true;
		itemsView.headers_visible = true;
		itemsView.hover_selection = true;

        TreeViewColumn column;

		//
		// Note Name Column
		//
		column = new Gtk.TreeViewColumn ();
		// Title for Column
		column.title = "Name";
		column.sizing = Gtk.TreeViewColumnSizing.AUTOSIZE;
		column.expand = true;
		column.resizable = false;

		Gtk.CellRendererPixbuf editRenderer = new Gtk.CellRendererPixbuf ();
		column.pack_start (editRenderer, false);
		column.set_cell_data_func (
			editRenderer,
			(cell_layout, cell, model, iter) =>	{
				Gtk.CellRendererPixbuf crp = cell as Gtk.CellRendererPixbuf;

        		IItem item;
				model.get (iter, 0, out item);
				if (item != null) {
					if (item.instanceOf == Utils.Constant.itemType.NOTE) 
						crp.pixbuf = Utils.UI.getPixbuf ("/icons/16x16/places/note.png" );
					else 
						crp.pixbuf = Utils.UI.getPixbuf ("/icons/16x16/actions/note-full.png" );

				}

        		
    		}
			);

		
		Gtk.CellRendererText nameRenderer = new Gtk.CellRendererText ();
		column.pack_start (nameRenderer, true);
		column.set_cell_data_func (
			nameRenderer,
			(cell_layout, cell, model, iter) =>	{
				Gtk.CellRendererText crt = cell as Gtk.CellRendererText;
				crt.ellipsize = Pango.EllipsizeMode.END;

        		IItem item;
				model.get (iter, 0, out item);
				if (item != null) crt.text = item.name;
    		}
			);
		nameRenderer.editable = false;
		//nameRenderer.edited += editNoteName;
		
		itemsView.append_column (column);


		//
		// Note UpdatedAt Column
		//
		column = new Gtk.TreeViewColumn ();
		// Title for Column
		column.title = "Last Changed";
		column.sizing = Gtk.TreeViewColumnSizing.FIXED;
		column.sizing = Gtk.TreeViewColumnSizing.AUTOSIZE;
		column.expand = true;
		column.resizable = true;
		
		Gtk.CellRendererText updateAtRenderer = new Gtk.CellRendererText ();
		column.pack_start (updateAtRenderer, true);
		updateAtRenderer.editable = false;
		column.set_cell_data_func 
			(updateAtRenderer,
			(cell_layout, cell, model, iter) =>	{
				Gtk.CellRendererText crt = cell as Gtk.CellRendererText;
				crt.ellipsize = Pango.EllipsizeMode.END;

	    		IItem item;
				model.get (iter, 0, out item);
				//if (item.backend != null)
					crt.text = item.updatedAt.to_string();
    		}
			);

		itemsView.append_column (column);	
	}

	private void updateIItemsModel() {
        Gtk.TreeIter ti;
        Gtk.ListStore treeModel = new Gtk.ListStore (1, typeof(IItem));

        /*
         unpack the notes and task list and store it in a tree model
        */
        foreach (IItem item in this.backend.notes) {
            treeModel.append (out ti);
            treeModel.set (ti, 0, item);
        }
        foreach (IItem item in this.backend.tasks) {
            treeModel.append (out ti);
            treeModel.set (ti, 0, item);
        }

 		filter = new Gtk.TreeModelFilter (treeModel, null);
		filter.set_visible_func( filterItemsTree );
	}

    private bool filterItemsTree (Gtk.TreeModel model, Gtk.TreeIter iter) {
 	    IItem item;
	    model.get (iter, 0, out item);
		if (item == null) return false;

		if (!item.isDeleted) return false;

		if (
				(
					(
						this.currentFilterRule == this.ALL
					) || (
						(
							this.currentFilterRule == this.NOTES
						) && (
							item.instanceOf == Utils.Constant.itemType.NOTE
						)
					) || (
						(this.currentFilterRule == this.TASKS) &&
						(item.instanceOf == Utils.Constant.itemType.TASK)
					) 
				) && (
					(searchString == "") || (searchString == Utils.Constant.filterRecover) ||
			  		(item.name.casefold().str(searchString.casefold()) != null) || 
			  		(item.text.casefold().str(searchString.casefold()) != null) 
				)
			
			) { 
			return true;
		} else {
			return false;
		}
   }
}
