
public class TaskMenuItem: Gtk.ImageMenuItem {

	public ITask task { get; set;}

	private Gtk.Label label;
	private Gdk.Pixbuf checked;
	private Gdk.Pixbuf unchecked;
	private Gdk.Pixbuf hover;	

	private Logger log;

	construct {
		checked = Utils.UI.getPixbuf("/icons/16x16/status/checkbox-marked.png");
		unchecked = Utils.UI.getPixbuf("/icons/16x16/status/checkbox-empty.png");
		hover = Utils.UI.getPixbuf("/icons/16x16/status/checkbox-active.png");
		log = Logger.getLogger("TaskMenuItem");
	}

	public TaskMenuItem (ITask task) {
			this.task = task;
            this.image = new Gtk.Image.from_pixbuf( (task.done) ? checked : unchecked);
			label = new Gtk.Label(Utils.UI.ellipsify(task.name,50));
			Gtk.Label due = new Gtk.Label("");
			due.use_markup = true;
			due.label = Utils.UI.getDueMarkup(task.dueAt, false, false);
			Gtk.Label spacer = new Gtk.Label("");

			Gtk.HBox box = new Gtk.HBox(false, 0);
			Gtk.Widget oldChild = this.child;
			remove(oldChild);
			add(box);

            Utils.UI.setTooltip(label, Utils.UI.ellipsify(this.task.text, 400));

			box.pack_start(oldChild, false, false, 0);
			box.pack_start(label, false, false, 0);
			box.pack_start(spacer, true, true, 0);
			box.pack_start(due, false, false, 0);
			this.show_all();			
	}

	public override bool motion_notify_event (Gdk.EventMotion ev) {
		Gtk.Image image = (this.image as Gtk.Image);
		if (!this.task.done && image != null) {
			if (ev.x >= image.allocation.x && 
				ev.x < image.allocation.x + image.allocation.width) {
				if (image.pixbuf != hover){
					image.pixbuf = hover;
				}
			} else if ( image.pixbuf != unchecked) {
				image.pixbuf = unchecked;
			}
		}
		return true;
	}

	public override bool button_press_event (Gdk.EventButton ev) {
		Gtk.Image image = (this.image as Gtk.Image);
		if (image != null) {
			if (ev.x >= image.allocation.x && 
				ev.x < image.allocation.x + image.allocation.width) {
				if (task.done){
					task.done = false;
					image.pixbuf = unchecked;
				} else {
					task.done = true;
					image.pixbuf = checked;
				}
				task.update();
				//FIXME show menu
				//(this.parent as AppletMenu).showMenu();
				//AppletMenu appletMenu = new AppletMenu(task.backend, icon);
	            //appletMenu.pop();
			} else {
				Utils.UI.showTaskWindow(task);
			}
		}
		return true;
	}

	public override bool leave_notify_event (Gdk.EventCrossing ev) {
		Gtk.Image image = (this.image as Gtk.Image);
		if (!this.task.done && image != null) {
			image.pixbuf = unchecked;
		}
		return true;
	}

}
