public class TaskWindow : Gtk.Window {

    private ITask task;
    private TextEditor textView;
	private IBackend backend;
    private Gtk.TreeModelFilter filter;

	Gtk.ToolButton projectsButton;
	Gtk.ToolButton categoriesButton;
	private string activeProjectID;
	private string activeCategoryID;

	public string id { get { return this.task.id;}}

    public TaskWindow (ITask aTask){
		this.task = aTask;
		this.backend = aTask.backend;
		activeProjectID = task.projectID;
		activeCategoryID = task.categoryID;

		createWindow();		

		//autosave at every 1 minute
		//TODO add config option
        GLib.Timeout.add_seconds(1 * 60 , () => {
			
			if ((this as TaskWindow) == null) return false; 
			this.updateTask(); 
			this.title=this.task.name; 
			return true;
			}
		);

	}

	private void createWindow()
	{

        this.title = this.task.name;
        this.icon = Utils.UI.getPixbuf ("/icons/16x16/status/checkbox-active.png" );

		if ( this.task.width < 50 )	this.task.width = 420;
		if ( this.task.height < 50 ) this.task.height = 250;
		this.default_width = this.task.width;
		this.default_height = this.task.height;

		/* update task when window is closed */
		this.delete_event += () => {
	   		Configuration.openTaskWindows.remove(this);
			updateTask();
			this.destroy();
		}; 


		/* text view widget */ 
        textView = new TextEditor(this.task);

		/* add task specific buttons to the toolbar*/
		populateToolbar();
		this.add(textView);

     }


    private void updateTask () {
		bool needs_update = false;

        int current_width;
        int current_height;
        this.get_size(out current_width, out current_height);
        if ((this.task.width != current_width) ||
            (this.task.height != current_height)) {
            this.task.width = current_width;
            this.task.height = current_height;
            needs_update = true;
        }

		if (this.task.name != this.textView.title) {
			if (this.textView.title != "") {
				this.task.name = this.textView.title;
			} else {
				this.task.name = _("Untitled Task");
			}
			needs_update = true;
		}

		if (this.task.text != this.textView.text) {
			this.task.text = this.textView.text;
			needs_update = true;
		}
		if (this.task.projectID != activeProjectID) {
			this.task.projectID = activeProjectID;
			needs_update = true;
		}

		if (this.task.categoryID != activeCategoryID) {
			this.task.categoryID = activeCategoryID;
			needs_update = true;
		}
		if (needs_update) this.task.update();
    }   


	private void showProjectsMenu(Gtk.Widget parent) {
		Gtk.Menu projectsMenu = new Gtk.Menu();
		projectsMenu.attach_to_widget(parent, () => {});

		GenericMenuItem item;
	    item = new GenericMenuItem("0", _("_No project") );
		if (activeProjectID == "0") item.active = true;
		item.activate += setProject;
	    projectsMenu.append (item);
   		Gtk.SeparatorMenuItem separator = new Gtk.SeparatorMenuItem();
        projectsMenu.append (separator);
		foreach (IProject project in this.backend.projects) {
			if (!project.isDeleted){
				item = new GenericMenuItem(project.id, project.name);
				item.activate += setProject;
				if (activeProjectID == project.id) item.active = true;
				projectsMenu.append (item);
			}			
		}
        projectsMenu.show_all();
        projectsMenu.popup (null, null, Utils.UI.getMenuPosition, 1, Gtk.get_current_event_time ());
	}

	private void setProject ( GenericMenuItem sender) {
		if (sender.id != this.activeProjectID) {
			activeProjectID = sender.id;
			projectsButton.set_label( (sender.id == "0") ? _("Project"): Utils.UI.ellipsify(sender.name,10));
		}
	}

	private void showCategoriesMenu(Gtk.Widget parent) {
		Gtk.Menu categoriesMenu = new Gtk.Menu();
		categoriesMenu.attach_to_widget(parent, () => {});

		GenericMenuItem item;
		foreach (ICategory category in this.backend.categories) {
			if (!category.isDeleted) {
				item = new GenericMenuItem(category.id, category.name);
				item.activate += setCategory;
				if (activeCategoryID == category.id) item.active = true;
				categoriesMenu.append (item);
			}			
		}
        categoriesMenu.show_all();
        categoriesMenu.popup (null, null, Utils.UI.getMenuPosition, 1, Gtk.get_current_event_time ());
	}


	private void setCategory ( GenericMenuItem sender) {
		if (sender.id != this.activeCategoryID) {
			activeCategoryID = sender.id;
			categoriesButton.set_label( (sender.id == "0") ? _("Category"): Utils.UI.ellipsify(sender.name,10));
		}
	}


    private void toggleDelete ( Gtk.ToolButton button ) {
		if (this.task.isDeleted) {
			this.task.isDeleted = false;
			/* FIXME don't create a new window when recovering the task*/
			this.task.push();
			button.set_icon_widget (new Gtk.Image.from_stock(Gtk.STOCK_DELETE, Gtk.IconSize.BUTTON));
			button.set_label(_("Delete"));
			button.show_all();

		} else {
			Configuration.openTaskWindows.remove(this);
		    this.task.delete();
			/* call the delete_event to do the closing part*/
		    this.delete_event(new Gdk.Event(Gdk.EventType.DELETE));
		}
    }

	private void toggleDone ( Gtk.ToolButton button ) {

		if (this.task.done) { 
			this.task.done = false;
			button.set_icon_widget (new Gtk.Image.from_pixbuf(Utils.UI.getPixbuf ("/icons/16x16/status/checkbox-empty.png" )));
			button.set_label(_("Done"));
		} else {
			this.task.done = true;
			button.set_icon_widget (new Gtk.Image.from_pixbuf( Utils.UI.getPixbuf ("/icons/16x16/status/checkbox-marked.png" )));	
			button.set_label(_("Not Done"));
		}
		button.show_all();

	}


	private void populateToolbar() {

		Gtk.ToolButton doneButton = (this.task.done) ? 
							new Gtk.ToolButton( new Gtk.Image.from_pixbuf(Utils.UI.getPixbuf ("/icons/16x16/status/checkbox-marked.png" )), _("Not Done")) :
							new Gtk.ToolButton( new Gtk.Image.from_pixbuf(Utils.UI.getPixbuf ("/icons/16x16/status/checkbox-empty.png" )), _("Done"));
		doneButton.clicked += toggleDone;
		doneButton.has_tooltip = true;
		doneButton.query_tooltip += (sender, x, y, keyboard_tooltip, tooltip) => {
			if (this.task.done)
				(tooltip as Gtk.Tooltip).set_text(_("Mark task as not done"));
			else
				(tooltip as Gtk.Tooltip).set_text(_("Mark task as done"));
			return true;
		};
		textView.toolbar.insert(doneButton, 0);
		
        categoriesButton = new Gtk.ToolButton( new Gtk.Image.from_pixbuf( Utils.UI.getPixbuf ("/icons/22x22/actions/bookmark-new.png" )),_("Category"));
		categoriesButton.clicked += (sender) => {showCategoriesMenu(sender);};
		categoriesButton.has_tooltip = true;
		categoriesButton.query_tooltip += (sender, x, y, keyboard_tooltip, tooltip) => {
			(tooltip as Gtk.Tooltip).set_text(_("Assign task to a category"));
			return true;
		};
		/* FIXME use a better contant */
		if (task.categoryID != "0") {
		    foreach (ICategory category in backend.categories) {
			    if (category.id == task.categoryID) {
				    categoriesButton.set_label(Utils.UI.ellipsify(category.name,10));
				    break;
			    }
		    }
		}
		textView.toolbar.insert(new Gtk.SeparatorToolItem (), -1);
		textView.toolbar.insert(categoriesButton, -1);


        projectsButton = new Gtk.ToolButton( new Gtk.Image.from_pixbuf( Utils.UI.getPixbuf ("/icons/22x22/actions/note-full.png" )),_("Project"));
		projectsButton.clicked += (sender) => {showProjectsMenu(sender);};
		projectsButton.has_tooltip = true;
		projectsButton.query_tooltip += (sender, x, y, keyboard_tooltip, tooltip) => {
			(tooltip as Gtk.Tooltip).set_text(_("Assign task to a project"));
			return true;
		};
		/* FIXME use a better contant */
		if (task.projectID != "0") {
		    foreach (IProject project in backend.projects) {
			    if (project.id == task.projectID) {
				    projectsButton.set_label(Utils.UI.ellipsify(project.name,10));
				    break;
			    }
		    }
        }
		textView.toolbar.insert(projectsButton, -1);

        Gtk.ToolButton deleteButton;


		if (this.task.isDeleted) {
	    	deleteButton = new Gtk.ToolButton(new Gtk.Image.from_stock(Gtk.STOCK_UNDO, Gtk.IconSize.BUTTON),_("Recover"));
		} else {
	        deleteButton = new Gtk.ToolButton(new Gtk.Image.from_stock(Gtk.STOCK_DELETE, Gtk.IconSize.BUTTON),_("Delete"));
		}
		deleteButton.clicked += toggleDelete;
		deleteButton.has_tooltip = true;
		deleteButton.query_tooltip += (sender, x, y, keyboard_tooltip, tooltip) => {
			if (this.task.isDeleted)
				(tooltip as Gtk.Tooltip).set_text(_("Recover this task"));
			else 
				(tooltip as Gtk.Tooltip).set_text(_("Delete this task"));
			return true;
		};
		textView.toolbar.insert(new Gtk.SeparatorToolItem (), -1);
		textView.toolbar.insert(deleteButton, -1);


        Gtk.EventBox edit_area = new Gtk.EventBox();
        Gtk.TreeIter ti;
        Gtk.ListStore task_model = new Gtk.ListStore(1, typeof(ITask));
        task_model.append (out ti);
        task_model.set (ti, 0, this.task);

 		filter = new Gtk.TreeModelFilter(task_model, null);
        // FIXME catch memory leaks
		this.task.updated.connect((obj) => {this.filter.refilter();});
        
        Gtk.TreeView task_box = new Gtk.TreeView();
        task_box.headers_visible = false;
		Gtk.TreeViewColumn column;
        task_box.set_model(filter);
		
		column = Utils.UI.makeDueDateColumn(task_box, edit_area);
		task_box.append_column(column);

		column = Utils.UI.makeDeferColumn(task_box, edit_area);
		task_box.append_column(column);
		
        textView.vbox.pack_start(task_box, true, true, 0);
        textView.vbox.pack_start(edit_area, false, false, 0);        
	}

/* end of TaskWindow */
}
