using Gtk;

public class TasksGroup : VBox {


    private Gtk.Bin editArea;
    private TasksTreeView treeView;
	private Gtk.TreeModelFilter filter;
	private int filterType;
	private string filterValue;
	private IBackend backend;
    public static enum groupType { HOME = 0, CALENDAR, TICKLER}
    public static string CALENDAR_TODAY = "today";
    public static string CALENDAR_WEEK_REST = "week-rest";
    public static string CALENDAR_WEEK_NEXT = "week-next";
    public static string CALENDAR_MONTH_REST = "month-rest";
    public static string CALENDAR_LATER = "later";
    
    public TasksGroup (string groupName, int groupType, string groupValue, IBackend aBackend, Gtk.Widget anEditArea) {
        editArea = (anEditArea as Gtk.Bin);
		backend = aBackend;
		filterType = groupType;
		filterValue = groupValue;
		create(groupName);
	}

	private void create (string groupName) {
        HBox headerHBox = new HBox (false, 0);
        Label header = new Label ("");
		header.use_markup = true;
		header.use_underline = false;
		header.xalign = 0;
		header.set_markup(getHeaderMarkup (groupName));

        headerHBox.pack_start (header, false, false, 0);
        Label spacer = new Label ("");
        headerHBox.pack_start (spacer, true, true, 0);
		pack_start (headerHBox, false, false, 0);

		treeView = new TasksTreeView (editArea, false);
		pack_start (treeView, true, true, 0);
		
    }

	public void update () {
		Gtk.TreeModel treeModel = Utils.UI.tasksToListStore(backend.tasks);
 		filter = new Gtk.TreeModelFilter (treeModel, null);
		filter.set_visible_func( filterFunc );
		treeView.model = this.filter ;
		this.filter.refilter();

	}

    private bool filterFunc (Gtk.TreeModel model, Gtk.TreeIter iter) {
		switch (filterType) {
			case (groupType.CALENDAR) :
				return filterFuncCalendar(model, iter);
			case (groupType.TICKLER) :
				return filterFuncTickler(model, iter);
			default :
				return filterFuncHome(model, iter);
		}
    }

    private bool filterFuncHome (Gtk.TreeModel model, Gtk.TreeIter iter) {
 	    ITask task;
	    model.get (iter, 0, out task);
		if ((task == null) || (task.isDeleted) || (task.done)) return false;
		if ((task.categoryID == filterValue) && (task.showFrom.mktime() <= time_t()))
			return true;
		else
			return false;
    }

    private bool filterFuncTickler (Gtk.TreeModel model, Gtk.TreeIter iter) {
 	    ITask task;
	    model.get (iter, 0, out task);
		if ((task == null) || (task.isDeleted) || (task.done)) return false;
		if ((task.categoryID == filterValue) && (task.showFrom.mktime() > time_t()))
			return true;
		else
			return false;
    }

    private bool filterFuncCalendar (Gtk.TreeModel model, Gtk.TreeIter iter) {
 	    ITask task;
	    model.get (iter, 0, out task);
		if ((task == null) || (task.isDeleted) || (task.done)) return false;

        GLib.Date start;
        GLib.Date end;

		/* today filter */
		if ( (filterValue == CALENDAR_TODAY) && (task.dueAt.mktime() > 0 ) && (task.dueAt.mktime() < time_t()))
			return true;

		/* Rest of this week */
		start = Utils.DateTime.newDateNow();
        end = Utils.DateTime.newDateNow();
        uint week = start.get_monday_week_of_year ();
        end.add_days( ((week+1) * 7) - week * 7 - 2 );

		if ( (filterValue == CALENDAR_WEEK_REST) && filterInterval(task,start,end))
			return true;

        /* Next week */
        start = end;
        end.add_days(6);
		if ( (filterValue == CALENDAR_WEEK_NEXT) && filterInterval(task,start,end))
			return true;


		/* Rest of this month */
        start = end;
        end.set_day((GLib.DateDay)(GLib.Date.get_days_in_month(start.get_month(), start.get_year())));
		if ( (filterValue == CALENDAR_MONTH_REST) && filterInterval(task,start,end))
			return true;

        /* Next month and later */
        start = end;
        end.add_years(1000);
		if ( (filterValue == CALENDAR_LATER) && filterInterval(task,start,end))
			return true;

		return false;

    }

	private bool filterInterval (ITask task, GLib.Date start, GLib.Date end) {
		if (task.dueAt.mktime() < 1) return false;

		GLib.Date taskDate;
		taskDate = Utils.DateTime.newDateFromTime( task.dueAt );
        start.subtract_days(1);
        end.add_days(1);
		if ((taskDate.compare(start) > 0 ) && (taskDate.compare(end) < 0 )) return true;

		return false;

	}

    private string getHighlightColor ()	{
        Gdk.Color fgColor;
        Gtk.Style style = Gtk.rc_get_style (this); 
        fgColor = style.bg [(int) StateType.SELECTED];
        return fgColor.to_string();
    }

    private string getHeaderMarkup (string groupName) {
        string ret =  "<span size=\"x-large\"  foreground=\"%s\" weight=\"bold\">%s</span>";
        return ret.printf(getHighlightColor (), groupName);
    }



}
