using Gtk;

public class TasksTreeView : TreeView {

    private Gtk.Bin editArea;
    private ITask editedTask;
	private bool showHeader;

    construct {
        editedTask = null;
    }

    public TasksTreeView(Gtk.Widget anEditArea, bool viewHeader) {
        editArea = (anEditArea as Gtk.Bin);
		showHeader = viewHeader;
		createView();
	}

	public override void row_activated (Gtk.TreePath path, Gtk.TreeViewColumn column) {
		Gtk.TreeIter iter;
		ITask task;
		if (!model.get_iter (out iter, path))  return; // Do nothing
		model.get (iter, 0, out task);	
		if ( task == null ) return;	 // if error .. do nothing
		Utils.UI.showTaskWindow(task);
    }

   	private void createView() {
		//selection.mode = Gtk.SelectionMode.SINGLE;
		rules_hint = true;
		
		//FIXME headers is gone
		headers_visible = (showHeader) ? true : false;
		hover_selection = true;

        TreeViewColumn column;

		//
		// Checkbox Column
		//
		column = new TreeViewColumn ();
		// Title for Completed/Checkbox Column
		column.title =  "";
		column.sizing = TreeViewColumnSizing.AUTOSIZE;
		column.resizable = false;
		column.clickable = true;
	
		Gtk.CellRendererToggle doneRenderer = new Gtk.CellRendererToggle ();
		doneRenderer.toggled += toggleTask;
		column.pack_start (doneRenderer, false);
		column.set_cell_data_func (doneRenderer, taskToggleCellDataFunc);
		append_column (column);
		//
		// Due Date Column
		//
		//  2/11 - Today
		//  2/12 - Tomorrow
		//  2/13 - Wed
		//  2/14 - Thu
		//  2/15 - Fri
		//  2/16 - Sat
		//  2/17 - Sun
		// --------------
		//  2/18 - In 1 Week
		// --------------
		//  No Date
		// ---------------
		//  Choose Date...
		
		column = Utils.UI.makeDueDateColumn(this, this.editArea);
		append_column (column);

		//
		// Defer Column
		//
		column = Utils.UI.makeDeferColumn(this, this.editArea);
		append_column (column);

		//
		// Task Name Column
		//
		column = new Gtk.TreeViewColumn ();
		// Title for Task Name Column
		column.title = "Name";
		column.sizing = Gtk.TreeViewColumnSizing.FIXED;
		column.sizing = Gtk.TreeViewColumnSizing.AUTOSIZE;
		column.expand = true;
		column.resizable = true;
		
		Gtk.CellRendererText nameRenderer = new Gtk.CellRendererText ();
		column.pack_start (nameRenderer, true);
		column.set_cell_data_func (nameRenderer, taskNameTextCellDataFunc);
		nameRenderer.editable = true;
		nameRenderer.edited += editTaskName;
		
		append_column (column);


		//
		// Edit Column
		//
		column = new TreeViewColumn ();
		// Title for Completed/Checkbox Column
		column.title =  "";
		column.sizing = TreeViewColumnSizing.AUTOSIZE;
		column.resizable = false;
		column.clickable = true;
		
		Gtk.CellRendererPixbuf editRenderer = new Gtk.CellRendererPixbuf ();
		column.pack_start (editRenderer, false);
		column.set_cell_data_func (editRenderer, taskEditCellDataFunc);
		append_column (column);
    }

    private void taskToggleCellDataFunc (Gtk.CellLayout cell_layout,
					Gtk.CellRenderer cell,
					Gtk.TreeModel model,
					Gtk.TreeIter iter)
   {
        Gtk.CellRendererToggle crt = cell as Gtk.CellRendererToggle;
        ITask task;
        this.model.get (iter, 0, out task );
        crt.active = task.done ? true: false;
    }

    private void taskNameTextCellDataFunc (Gtk.CellLayout cell_layout,
					    Gtk.CellRenderer cell,
					    Gtk.TreeModel model,
					    Gtk.TreeIter iter)	{
		Gtk.CellRendererText crt = cell as Gtk.CellRendererText;
		crt.ellipsize = Pango.EllipsizeMode.END;

		ITask task;
		model.get (iter, 0, out task);	
		crt.text = task.name;
    }


    private void taskEditCellDataFunc (Gtk.CellLayout cell_layout,
					Gtk.CellRenderer cell,
					Gtk.TreeModel model,
					Gtk.TreeIter iter)	{
		Gtk.CellRendererPixbuf crp = cell as Gtk.CellRendererPixbuf;

		ITask task;
		model.get (iter, 0, out task);	
		if (task.text == "") {
			crp.pixbuf = Utils.UI.getPixbuf ("/icons/22x22/actions/note-blank.png" );
		} else {
			crp.pixbuf = Utils.UI.getPixbuf ("/icons/22x22/actions/note-full.png" );
		}


    }

    private void toggleTask ( Gtk.CellRendererToggle sender, string path) {
        Gtk.TreeIter iter;
        ITask task = null;
        Gtk.TreePath aPath = new Gtk.TreePath.from_string( path );

        if (!this.model.get_iter (out iter, aPath))
            return; // Do nothing
        this.model.get (iter, 0, out task);
        if ( task == null )
            return;
		task.done = (task.done) ? false : true;
        task.update();
    }

    private void editTaskName ( GLib.Object sender, string path, string newName) {
        Gtk.TreeIter iter;
        ITask task;
        
        Gtk.TreePath aPath = new Gtk.TreePath.from_string( path );
        if (!this.model.get_iter (out iter, aPath))
	    return; // Do nothing
				
		model.get (iter, 0, out task);	
        if ( task == null )
            return;		
		if (task.name != newName) {
            task.name = newName;
            task.update();       
        }
    }

 }
