public class TextEditor : Gtk.VBox {

	private string _content {
		get { 
				Gtk.TextIter start;
				Gtk.TextIter end;
				textBuffer.get_start_iter( out start );
				textBuffer.get_end_iter( out end );
				return textBuffer.get_text(start, end, true);
			}
		set {
				textBuffer.begin_not_undoable_action();
            	textBuffer.set_text(value, (int)value.size());
				textBuffer.end_not_undoable_action();
			}
	}

	private string _title;
	public string title {
		get {
				string[] lines = this._content.split("\n",2);
				_title = (lines[0] != null) ? lines[0] : "";
				return _title;
			}
	}

	private string _text;
	public string text {
		get {
				string[] lines = this._content.split("\n",2);
				_text = lines[1];
				return _text;
			}
	}

	public Gtk.Toolbar toolbar {get; set;}
    public Gtk.VBox vbox {get; set;}
    
	public string id { get; set;}

	private Logger log;
	private Gtk.EventBox searchBox;
	private Gtk.Toolbar searchBar;
	private Gtk.Entry searchEntry;
	private Gtk.TextIter matchStart;
	private Gtk.TextIter matchEnd;

	private Gtk.SourcePrintCompositor compositor;

    private Gtk.TextTag titleTag;
    private Gtk.TextTag searchTag;

	private Gtk.TextTagTable tagTable;
	private Gtk.SourceBuffer textBuffer;
	private Gtk.SourceView textView;
	private Gtk.Spell spell;

	construct {
		log = Logger.getLogger("TextEditor");


		toolbar = new Gtk.Toolbar();
        vbox = new Gtk.VBox(false,0);
		tagTable = new Gtk.TextTagTable();
		textBuffer = new Gtk.SourceBuffer(tagTable);
		textBuffer.changed += () => {setTitleTag();};

		titleTag = textBuffer.create_tag("title",
				"scale", Pango.Scale.XX_LARGE,
				"foreground", "royalblue", //FIXME use __gdk_color_constrast function, see Tomboy
				"underline", Pango.Underline.SINGLE,
				null);

		searchTag = textBuffer.create_tag("search",
				"background", "#FFFF00", //FIXME use __gdk_color_constrast function, see Tomboy
				null);

		textView = new Gtk.SourceView.with_buffer(textBuffer);
		textView.populate_popup += populateMenu;
        textView.left_margin = 5;
		textView.right_margin = 5;
		textView.wrap_mode = Gtk.WrapMode.WORD;
		textView.editable = true;
		textView.can_focus = true;
		textView.set_highlight_current_line(true);

		spell = new Gtk.Spell.attach(textView,null);

		searchBox = new Gtk.EventBox();
	}

	public TextEditor (IItem item) {
		this.id = item.id;
		this._content = item.name + "\n" + item.text;

		initSearchBar();
		populateToolbar();

        Gtk.Adjustment hScrollAdjustment = new Gtk.Adjustment (0,0,0,0,0,0);
        Gtk.Adjustment vScrollAdjustment = new Gtk.Adjustment (0,0,0,0,0,0);
        Gtk.ScrolledWindow scrolledWindow = new Gtk.ScrolledWindow (hScrollAdjustment, vScrollAdjustment);
        scrolledWindow.hscrollbar_policy = Gtk.PolicyType.AUTOMATIC;
        scrolledWindow.vscrollbar_policy = Gtk.PolicyType.AUTOMATIC;
        scrolledWindow.add(textView);

        vbox.pack_start(toolbar, true, true, 0);
		pack_start(vbox, false , false, 0);
    	pack_start(scrolledWindow, true, true, 0);
		pack_start(searchBox, false, false, 0);
		setTitleTag();
	}

/*****************************************************************************
 *
 */

	private void setTitleTag() {
			Gtk.TextIter start;
			Gtk.TextIter end;
			textBuffer.get_iter_at_line( out start,0);
			textBuffer.get_iter_at_line( out end,1);
			textBuffer.apply_tag_by_name("title",start,end);
	}



	private void populateToolbar() {

		Gtk.ToolButton searchButton = new Gtk.ToolButton( new Gtk.Image.from_stock(Gtk.STOCK_FIND, Gtk.IconSize.BUTTON), _("Search"));
		searchButton.clicked.connect(showSearch);
		searchButton.has_tooltip = true;
		searchButton.query_tooltip += (sender, x, y, keyboard_tooltip, tooltip) => {
			(tooltip as Gtk.Tooltip).set_text(_("Search within this note"));
			return true;
		};
		toolbar.insert(searchButton, -1);


		Gtk.ToolButton editButton = new Gtk.ToolButton( new Gtk.Image.from_stock(Gtk.STOCK_SELECT_FONT, Gtk.IconSize.BUTTON), _("Edit"));
		editButton.clicked.connect(showEditMenu);
		editButton.has_tooltip = true;
		editButton.query_tooltip += (sender, x, y, keyboard_tooltip, tooltip) => {
			(tooltip as Gtk.Tooltip).set_text(_("Text editing related actions."));
			return true;
		};
		toolbar.insert(editButton, -1);

		Gtk.ToolButton actionsButton = new Gtk.ToolButton( new Gtk.Image.from_stock(Gtk.STOCK_EXECUTE, Gtk.IconSize.BUTTON), _("Tools"));
		actionsButton.clicked.connect(showToolsMenu);
		actionsButton.has_tooltip = true;
		actionsButton.query_tooltip += (sender, x, y, keyboard_tooltip, tooltip) => {
			(tooltip as Gtk.Tooltip).set_text(_("Various actions and tools."));
			return true;
		};
		toolbar.insert(actionsButton, -1);
       
	}

	public void populateMenu (Gtk.TextView textView, Gtk.Menu menu) {
        /* right now there are no special menu items */
	}

	public void showSearch() {
		if (searchBox.child == null) {
			searchBox.add(searchBar);
			searchBox.show_all();
			searchEntry.grab_focus();
			doSearchInit();
		}
	}

	private void removeSearchTag() {
		Gtk.TextIter start;
		Gtk.TextIter end;
		textBuffer.get_start_iter( out start);
		textBuffer.get_end_iter( out end);
		textBuffer.remove_tag_by_name("search", start, end);
	}

	private void initSearchBar () {
		if (searchBar == null ) { /* init searchFields */
			searchBar = new Gtk.Toolbar();
			searchBar.set_style(Gtk.ToolbarStyle.BOTH_HORIZ);

			Gtk.Label searchText = new Gtk.Label(_("Search:"));
			searchEntry = new Gtk.Entry();

            /* Label for search next match in text.*/
			Gtk.ToolButton searchNext = new Gtk.ToolButton( new Gtk.Image.from_stock(Gtk.STOCK_GO_FORWARD, Gtk.IconSize.SMALL_TOOLBAR), _("Next"));
			searchNext.clicked.connect(doSearchNext);
			searchNext.has_tooltip = true;
			searchNext.query_tooltip += (sender, x, y, keyboard_tooltip, tooltip) => {
				(tooltip as Gtk.Tooltip).set_text(_("Find next occurance"));
				return true;
			};

            /* Label for search previous match in text.*/
			Gtk.ToolButton searchPrev = new Gtk.ToolButton( new Gtk.Image.from_stock(Gtk.STOCK_GO_BACK, Gtk.IconSize.SMALL_TOOLBAR), _("Previous"));
			searchPrev.clicked.connect(doSearchPrev);
			searchPrev.has_tooltip = true;
			searchPrev.query_tooltip += (sender, x, y, keyboard_tooltip, tooltip) => {
				(tooltip as Gtk.Tooltip).set_text(_("Find previous occurance"));
				return true;
			};
		
			Gtk.ToolItem searchItem = new Gtk.ToolItem();

			Gtk.HBox hBox = new Gtk.HBox(false, 0);
			hBox.pack_start(searchText, false, false, 0);
			hBox.pack_start(searchEntry, false, false, 0);
			searchItem.add(hBox);

			searchBar.insert(searchItem, -1);
			searchBar.insert(searchPrev, -1);
			searchBar.insert(searchNext, -1);
			
			searchEntry.focus_out_event.connect( ( ) => {
				removeSearchTag();
                this.searchBox.remove(this.searchBox.get_child());        
            });
			searchEntry.changed.connect(doSearchInit);
		}
	}

	private void doSearchNext(Gtk.Widget sender) {
		doSearch(this.searchEntry.get_text(), matchEnd, true);
	}

	private void doSearchPrev(Gtk.Widget sender) {
		doSearch(this.searchEntry.get_text(), matchStart, false);
	}

	private void doSearchInit() {
		Gtk.TextIter searchStart;
		this.textBuffer.get_iter_at_line( out searchStart, 0);
		doSearch(this.searchEntry.get_text(),searchStart, true);
	}

	private void doSearch(string text, Gtk.TextIter searchStartIter, bool forward) {

		bool found;
		if (forward) {
			found = Gtk.source_iter_forward_search (
				searchStartIter, 
				text, 
				Gtk.SourceSearchFlags.CASE_INSENSITIVE | Gtk.SourceSearchFlags.TEXT_ONLY, 
				out matchStart, 
				out matchEnd, 
				null
			);
		} else {
			found = Gtk.source_iter_backward_search (
				searchStartIter, 
				text, 
				Gtk.SourceSearchFlags.CASE_INSENSITIVE | Gtk.SourceSearchFlags.TEXT_ONLY, 
				out matchStart, 
				out matchEnd, 
				null
			);
		}

		if (found) {
			removeSearchTag();
			textBuffer.place_cursor(matchStart);
			textView.scroll_to_iter(matchStart, 0, false, 0, 0);
			textBuffer.apply_tag_by_name("search", matchStart, matchEnd);
		}
	}

	private void showEditMenu(Gtk.Widget parent) {
		Gtk.Menu editMenu = new Gtk.Menu();
		editMenu.attach_to_widget(parent, () => {});

		Gtk.ImageMenuItem undo = new Gtk.ImageMenuItem.from_stock (Gtk.STOCK_UNDO, null);
		undo.sensitive = textBuffer.can_undo;
		undo.activate.connect(textBuffer.undo);

		Gtk.ImageMenuItem redo = new Gtk.ImageMenuItem.from_stock (Gtk.STOCK_REDO, null);
		redo.sensitive = textBuffer.can_redo;
		redo.activate.connect(textBuffer.redo);

		Gtk.SeparatorMenuItem separator = new Gtk.SeparatorMenuItem();	
	    editMenu.append (undo);	
		editMenu.append (redo);		
		editMenu.append (separator);

        editMenu.show_all();
        editMenu.popup (null, null, Utils.UI.getMenuPosition, 1, Gtk.get_current_event_time ());
	}

	private void showToolsMenu(Gtk.Widget parent) {
		Gtk.Menu toolsMenu = new Gtk.Menu();
		toolsMenu.attach_to_widget(parent, () => {});

		Gtk.ImageMenuItem print = new Gtk.ImageMenuItem.from_stock (Gtk.STOCK_PRINT, null);
		Gtk.ImageMenuItem saveTXT = new Gtk.ImageMenuItem.with_mnemonic (_("Save as _TXT"));
		Gtk.ImageMenuItem saveHTML = new Gtk.ImageMenuItem.with_mnemonic (_("Save as _HTML"));
		print.activate.connect(doPrint);
		Gtk.SeparatorMenuItem separator = new Gtk.SeparatorMenuItem();	
	    toolsMenu.append (print);
		toolsMenu.append (saveTXT);
		toolsMenu.append (saveHTML);	
		toolsMenu.append (separator);

        toolsMenu.show_all();
        toolsMenu.popup (null, null, Utils.UI.getMenuPosition, 1, Gtk.get_current_event_time ());
	}

	private void doPrint() {
		compositor = new Gtk.SourcePrintCompositor.from_view(textView);
		Gtk.PrintOperation operation = new Gtk.PrintOperation();	

		operation.paginate.connect(printPaginate);
		operation.draw_page.connect(printDrawPage);
		try {
			operation.run( Gtk.PrintOperationAction.PRINT_DIALOG, null);
		} catch( GLib.Error ex ) {
			log.error("Error running print dialog: %s".printf(ex.message));
		}
	}

	private bool printPaginate(Gtk.PrintOperation operation, Gtk.PrintContext context) {
		if (compositor.paginate (context)) {
			int pages;
			pages = compositor.get_n_pages ();
			operation.set_n_pages (pages);
			return true;
		}

		return false;
	}

	private void printDrawPage(Gtk.PrintOperation operation, Gtk.PrintContext context, int pageNr) {
		compositor.draw_page(context, pageNr);

	}
/* end of TextEditor */
}
