using Gtk;

public class TicklerView :  Gtk.ScrolledWindow {

	private Logger log;
	private IBackend backend;
    private Gtk.Widget editArea;
	private List<TasksGroup> groups;

	construct {
		log = Logger.getLogger("TicklerView");
	}

    public TicklerView (IBackend aBackend, Gtk.Widget anEditArea)
    {
		backend = aBackend;
		editArea = anEditArea;
		create();
	}


	public void updateAll () {
		Gtk.Widget? old = this.get_child();
		if (old != null) this.remove(old);				
		create();
		update();
	}

	public void update() {
		foreach (TasksGroup group in groups) {
			group.update();
		}
	}


	private void create() {    
        Gtk.VBox box = new Gtk.VBox(false,0);
		groups = new List<TasksGroup>();

        TasksGroup categoriesGroup;
        foreach (ICategory category in this.backend.categories) {
			if (!category.isDeleted) {
				Gee.ArrayList<ITask> tasks;
				tasks = Utils.Backend.getTasksByCategory(this.backend.tasks, category.id , false, true);
				categoriesGroup = new TasksGroup(category.name, TasksGroup.groupType.TICKLER, category.id, backend, this.editArea);
				groups.append(categoriesGroup);

				box.pack_start(categoriesGroup,false,false,0);
			}
         }

       //add end spacer
        Label endSpacer = new Label("");
        box.pack_start(endSpacer,true,true,0);
        
        Adjustment hScrollAdjustment = new Adjustment (0,0,0,0,0,0);
        Adjustment vScrollAdjustment = new Adjustment (0,0,0,0,0,0);
		set_hadjustment(hScrollAdjustment);
		set_vadjustment(vScrollAdjustment);
        this.hscrollbar_policy = PolicyType.NEVER;
        this.vscrollbar_policy = PolicyType.AUTOMATIC;

		Gtk.EventBox evBox = new Gtk.EventBox();

		Gdk.Color backgroundColor = getBackgroundColor ();
		evBox.modify_bg (StateType.NORMAL, backgroundColor);
		evBox.modify_base (StateType.NORMAL, backgroundColor);
		evBox.add(box);
		

        add_with_viewport(evBox);
 

    }

    private Gdk.Color getBackgroundColor () {
		Gtk.Style style = Gtk.rc_get_style (this);
		Gdk.Color color = style.base [(int)StateType.NORMAL];
		return color;
    }


}
