public class Utils.Backend {

    public static Gee.ArrayList<ITask> getTasksByCategory(Gee.ArrayList<ITask> tasks, string categoryID, bool onlyActive, bool onlyDeferred) {
        Gee.ArrayList<ITask> list = new Gee.ArrayList<ITask> ();
         
         foreach (ITask task in tasks ) {

	    if 	(
			!task.done &&
		    task.categoryID == categoryID && 
		    (	!onlyActive || 
			(onlyActive && (task.showFrom.mktime() <= time_t()))
		    ) &&
		    (	!onlyDeferred || 
			(onlyDeferred && (task.showFrom.mktime() > time_t()))
		    )
		)
	    {
                list.add (task);
            }
        }            
        return list;
	}


    /*
     task filtering functions
    */
    public static Gee.ArrayList<ITask> getTasksByToday( IBackend backend, bool onlyActive ) {
        Gee.ArrayList<ITask> list = new Gee.ArrayList<ITask> ();
         foreach (ITask task in backend.tasks) {
            time_t now = time_t ();
			if 	(
				((task.dueAt.mktime() > 0 ) && (task.dueAt.mktime() < now))  && 
				(	!onlyActive || 
				(onlyActive && (task.showFrom.mktime() <= time_t()))
				) && (!task.done)
			) {
		            list.add (task);
		    }
        }
        return list;
    }

    public static Gee.ArrayList<ITask> getTasksByTimeInteval( IBackend backend, GLib.Date start, GLib.Date end , bool onlyActive) {
	Gee.ArrayList<ITask> list = new Gee.ArrayList<ITask> ();
        GLib.Date taskDate;

        start.subtract_days(1);
        end.add_days(1);

        foreach (ITask task in backend.tasks) {
            taskDate = Utils.DateTime.newDateFromTime( task.dueAt );
            //if ((taskDate.compare(start) > 0 ) && (taskDate.compare(end) < 0 ) ) 
			if 	(
				((taskDate.compare(start) > 0 ) && (taskDate.compare(end) < 0 )) && 
				(	!onlyActive || 
				(onlyActive && (task.showFrom.mktime() <= time_t()))
				) && (!task.done)
			)	    
			{
                list.add (task);
            }
        }
        return list;
	}

    public static Gee.ArrayList<ITask> getTasksByProject(IBackend backend, string projectID, bool onlyActive) {
        Gee.ArrayList<ITask> list = new Gee.ArrayList<ITask> ();
         
         foreach (ITask task in backend.tasks ) {
	    if 	(
			!task.done &&
		    task.projectID == projectID && 
		    (	!onlyActive || 
			(onlyActive && (task.showFrom.mktime() <= time_t()))
		    )
		)	    
	    {
                list.add (task);
            }
        }                    
        return list;
	}

    public static void sortNotesByUpdatedAtDesc(Gee.ArrayList<INote> notes) {
        Utils.List.sort(notes , ( list, a, b) => {
            INote first = (INote)list.get(a);
            INote second = (INote)list.get(b);
			if (first.updatedAt.mktime() > second.updatedAt.mktime() ) {
				return -1;
			} else if (first.updatedAt.mktime() < second.updatedAt.mktime() ) {
				return 1;
	    	} else {
				return 0;
			}
        });
    }

    public static void sortProjectsByPositionAsc(Gee.ArrayList<IProject> projects) {
        Utils.List.sort(projects , ( list, a, b) => {
            IProject first = (IProject)list.get(a);
            IProject second = (IProject)list.get(b);
			if (first.position == second.position) {
				return 0;
			} else if (first.position < second.position) {
				return -1;
			} else {
				return 1;
			}
        });
    }

    public static void sortCategoriesByPositionAsc(Gee.ArrayList<ICategory> categories) {
        Utils.List.sort(categories , ( list, a, b) => {
            ICategory first = (ICategory)list.get(a);
            ICategory second = (ICategory)list.get(b);
			if (first.position == second.position) {
				return 0;
			} else if (first.position < second.position) {
				return -1;
			} else {
				return 1;
			}
        });
    }


}
