public class Utils.UI : GLib.Object {

    public static enum CellType { DEFER = 0, DUE_DATE}

    public static Gdk.Pixbuf getPixbuf ( string name ) {

        Gdk.Pixbuf pix = null;
        try {
            pix = new Gdk.Pixbuf.from_file( Config.PACKAGE_DATADIR + name );
        } catch( GLib.Error ex ) {
            stdout.printf("%s",ex.message); stdout.flush();
        }
        return pix;
    }

    public static void showErrorDialog ( Gtk.Window? parent, string message ) {
        Gtk.MessageDialog dialog = new Gtk.MessageDialog( parent , 
                                    Gtk.DialogFlags.DESTROY_WITH_PARENT,
	                                Gtk.MessageType.ERROR, 
                                    Gtk.ButtonsType.CLOSE,  message);

        dialog.run();
        dialog.destroy();
    }
      
    public static void showInfoDialog (Gtk.Window? parent, string message ) {
        
         Gtk.MessageDialog dialog = new Gtk.MessageDialog( parent , 
                                    Gtk.DialogFlags.DESTROY_WITH_PARENT,
	                                Gtk.MessageType.INFO, 
                                    Gtk.ButtonsType.CLOSE,  message);

        
        dialog.run();
        dialog.destroy();
     }

    public static void setTooltip(Gtk.Widget widget, string text){
        if (text.len() < 1) {
            return;
        }
		widget.has_tooltip = true;
	    widget.query_tooltip.connect( (sender, x, y, keyboard_tooltip, tooltip) => {
		    (tooltip as Gtk.Tooltip).set_text(text);
		    return true;
	    });
    }

	public static Gtk.ComboBox createProjectsComboBoxFromList (Gee.ArrayList<IProject> projects) {
	    Gtk.ComboBox combo = createProjectsComboBox ();
	    updateProjectsComboBox(combo, projects);
	    return combo;
    }

	public static Gtk.ComboBox createCategoriesComboBoxFromList (Gee.ArrayList<ICategory> categories) {
	    Gtk.ComboBox combo = createCategoriesComboBox ();
	    updateCategoriesComboBox(combo, categories);
	    return combo;
    }


    public static Gtk.ComboBox createProjectsComboBox () {
        Gtk.TreeIter ti;
	    Gtk.ComboBox combo = new Gtk.ComboBox ();
	    
        Gtk.CellRenderer renderer = new Gtk.CellRendererText();
        combo.pack_start(renderer, true);
        combo.add_attribute(renderer, "markup", 0);
        // set an initial list of projects... saying "Loading..." 
        Gtk.ListStore projectTemp = new Gtk.ListStore(1, typeof(string));
        projectTemp.append (out ti);
        projectTemp.set (ti, 0, _("Loading..."));
        combo.set_model(projectTemp);
        combo.set_active(0);
         
    	return combo;
    }


    public static void updateProjectsComboBox ( Gtk.ComboBox combo, Gee.ArrayList<IProject> projectsBack ) {
                 
        Gtk.TreeIter til;
        Gtk.ListStore projects = new Gtk.ListStore( 2, typeof( string ), typeof( int )); 
        projects.append (out til);
        projects.set (til, 0, "", 1, 0);

        foreach ( IProject project in projectsBack) {   
            projects.append (out til);
            projects.set (til, 0, project.name, 1, project.id);
        }
        combo.set_model(projects);
        combo.set_active(0);
    }

    public static Gtk.ComboBox createCategoriesComboBox () {
        Gtk.TreeIter ti;
	Gtk.ComboBox combo = new Gtk.ComboBox ();
	    
        Gtk.CellRenderer renderer = new Gtk.CellRendererText();
        combo.pack_start(renderer, true);
        combo.add_attribute(renderer, "markup", 0);
        // set an initial list of projects... saying "Loading..." 
        Gtk.ListStore projectTemp = new Gtk.ListStore(1, typeof(string));
        projectTemp.append (out ti);
        projectTemp.set (ti, 0, _("Loading..."));
        combo.set_model(projectTemp);
        combo.set_active(0);
    	return combo;
    }


    public static void updateCategoriesComboBox ( Gtk.ComboBox combo, Gee.ArrayList<ICategory> categoriesBack ) {
        Gtk.TreeIter til;
        Gtk.ListStore categories = new Gtk.ListStore (2, typeof (string), typeof (int)); 
        foreach ( ICategory category in categoriesBack) {   
            categories.append (out til);
            categories.set (til, 0, category.name, 1, category.id);
        }
        combo.set_model(categories);
        combo.set_active(0); // first category in order is the default one
        return;
    }

	public static void addNote ( IBackend backend) {
		INote note = backend.newNote();

		note.created += Utils.UI.noteCreatedCB;
		note.push();
	}

	public static void noteCreatedCB (INote note, bool status) {
		if (status) {
			note.name = _("New Note");
			note.text = "\n\n";
			note.update();
			Utils.UI.showNoteWindow(note);
		} else {
			Utils.UI.showErrorDialog( null ,("Backend error!\nCould not create new note."));
		}
	}

	public static void addTask ( IBackend backend) {
		ITask task = backend.newTask();
		ICategory defaultCategory = backend.categories.get(0);
		if (defaultCategory != null) task.categoryID = defaultCategory.id;
		task.created += (task, status) => {
			if (status) {
				task.name = _("New Task");
				task.update();
				Utils.UI.showTaskWindow(task);
			} else {
				Utils.UI.showErrorDialog( null ,("Backend error!\nCould not create new task."));
			}
		};
		task.push();
	}

	public static string ellipsify (string text, int max) {
		if (text.len() > (max+2) )
			return text.substring(0, max - 1) + "...";
		else
			return text;
	}

	public static string getDueMarkup (GLib.Time time, bool isBold, bool isShort) {
		string bg_color = "#FFFFFF";
		string out = "";
		string bold = (isBold) ? " weight=\"bold\"" : ""; 


       	if (time.mktime() > 0) {
            GLib.Date nowDate = Utils.DateTime.newDateNow();
            GLib.Date taskDate = Utils.DateTime.newDateFromTime(time);
            if (!taskDate.valid()) return "";
            int days = nowDate.days_between(taskDate);
            /* FIXME see how to deal with timezone */

            if ( days < 0 ) {
                bg_color = "#FF0000";
                out = "<span foreground=\"white\" background=\"%s\"%s> ".printf(bg_color, bold);
				out += (isShort) ?
					ngettext("Past %d day", "Past %d days",(days * -1)).printf((days * -1)):
					ngettext("Overdue by %d day", "Overdue by %d days",(days * -1)).printf((days * -1));
                out += " </span>";
            } else if ( days == 0 ) {
                bg_color = "#FF6600";
                out = "<span foreground=\"white\" background=\"%s\"%s> ".printf(bg_color, bold);
				out += (isShort) ? _("Today") : _("Due Today");
				out += " </span>";
            } else if ( days == 1 ) {
                bg_color = "#FF6600";
                out = "<span foreground=\"white\" background=\"%s\"%s> ".printf(bg_color, bold);
				out += (isShort) ? _("Tomorrow") : _("Due Tomorrow");
				out += " </span>";
            } else {
                bg_color = ( days > 7 ) ? "#33CC00" : "#FFA500";
                out = "<span foreground=\"white\" background=\"%s\"%s> ".printf(bg_color, bold);
				out += (isShort) ?
					ngettext("%d day", "%d days", days).printf( days ):
					ngettext("Due in %d day", "Due in %d days", days).printf( days );
				out += " </span>";
            }
		} else {
			return "";
		}
		return out;
	}

	public static string getDeferMarkup (GLib.Time time, bool isBold, bool isShort) {
		string out = "";
        string bg_color = "#FFFFFF";            
		string bold = (isBold) ? " weight=\"bold\"" : ""; 


		if (time.mktime() > 0) {
            GLib.Date nowDate = Utils.DateTime.newDateNow();
            GLib.Date taskDate = Utils.DateTime.newDateFromTime(time);
            if (!taskDate.valid()) return "";
            int days = nowDate.days_between(taskDate);
            /* FIXME see how to deal with timezone */
                
            if ( days < 0 ) {
                bg_color = "#FF0000";
                out = "<span foreground=\"white\" background=\"%s\"%s> ".printf(bg_color, bold);
				out += (isShort) ?
					ngettext("Active %d day", "Active %d days",(days * -1)).printf((days * -1)):
					ngettext("Active since %d day", "Active since %d days",(days * -1)).printf((days * -1));
				out += " </span>";
				return "";
            } else if ( days == 0 ) {
                bg_color = "#FF6600";
                out = "<span foreground=\"white\" background=\"%s\"%s> ".printf(bg_color, bold);
				out += (isShort) ? _("Today") : _("Show Today");
				out += " </span>";
            } else if ( days == 1 ) {
                bg_color = "#FF6600";
                out = "<span foreground=\"white\" background=\"%s\"%s> ".printf(bg_color, bold);
				out += (isShort) ? _("Tomorrow") : _("Show Tomorrow");
				out += " </span>";
            } else {
                bg_color = ( days > 7 ) ? "#33CC00" : "#FFA500";
                out = "<span foreground=\"white\" background=\"%s\"%s> ".printf(bg_color, bold);
				out += (isShort) ?
                         ngettext("%d day", "%d days", days).printf( days ):
                         ngettext("Show in %d day", "Show in %d days", days).printf( days );
				out += " </span>";
            }
        }
		return out;
	}

	public static Gtk.ListStore tasksToListStore (Gee.ArrayList<ITask> tasks) {
        Gtk.TreeIter ti;
        Gtk.ListStore treeModel = new Gtk.ListStore (1, typeof(ITask));
        /*
         unpack the task list and store it in a tree model
        */
        foreach (ITask task in tasks) {
            treeModel.append (out ti);
            treeModel.set (ti, 0, task);
        }
		return treeModel;
	}

    public static void getMenuPosition (Gtk.Menu menu, out int x, out int y, out bool push_in) {
    	// FIXME replace get_parent_window() with "window" --- problems on janty
        if (menu.get_attach_widget() == null || menu.get_attach_widget().get_parent_window() == null) {
            // Prevent null exception in weird cases
            x = 0;
            y = 0;
            push_in = true;
            return;
        }

        menu.get_attach_widget().get_parent_window().get_origin (out x, out y);
        x += menu.get_attach_widget().allocation.x;
    
        int width;
        int height;
        menu.get_size_request (out width, out height);
        if (y + height >= menu.get_attach_widget().get_screen().height())
            y -= height;
        else
            y += menu.get_attach_widget().allocation.height;
        push_in = true;
    }

	public static void showNoteWindow(INote note) {
		NoteWindow noteWindow = null;
		foreach (NoteWindow win in Configuration.openNoteWindows) {
			if (win.id == note.id ) {
				noteWindow =  win;
				break;
			}
		}
		if (noteWindow == null) {
			NoteWindow win = new NoteWindow(note);
			win.show_all();
			Configuration.openNoteWindows.add(win);
		} else {
			//FIXME just raise the window
			noteWindow.hide_all();
			noteWindow.show_all();
		}
	}

	public static void showTaskWindow(ITask task) {
		TaskWindow taskWindow = null;
		foreach (TaskWindow win in Configuration.openTaskWindows) {
			if (win.id == task.id ) {
				taskWindow =  win;
				break;
			}
		}
		if (taskWindow == null) {
			TaskWindow win = new TaskWindow(task);
			win.show_all();
			Configuration.openTaskWindows.add(win);
		} else {
			//FIXME just raise the window
			taskWindow.hide_all();
			taskWindow.show_all();
		}
	}


    public static Gtk.TreeViewColumn makeDueDateColumn(Gtk.TreeView tree_view, Gtk.Bin edit_area){

		Gtk.ListStore dueDateStore = new Gtk.ListStore (2, typeof (string), typeof (int));
		Gtk.TreeIter ti;
        Gtk.TreeViewColumn column = new Gtk.TreeViewColumn ();

		// Title for Due Date Column
		column.title = "Due";
		column.sizing = Gtk.TreeViewColumnSizing.AUTOSIZE;
		column.alignment = 0f;
		column.resizable = false;
		column.clickable = true;

		dueDateStore.append (out ti); 
        /* Menu entry to cancel a due date action. */
        dueDateStore.set (ti, 0, _("Nevermind"), 1, -3);

        /* Menu entry to set due date for today.*/
		dueDateStore.append (out ti); 
        dueDateStore.set (ti, 0, _("Today"), 1, 0);

        /* Menu entry to set due date for tomorrow.*/
		dueDateStore.append (out ti); 
        dueDateStore.set (ti, 0, _("Tomorrow"), 1, 1);

        /* Menu entry to set due date for the day after tomorrow.*/
		dueDateStore.append (out ti); 
        dueDateStore.set (ti, 0, _("Day after tomorrow"), 1, 2);

        /* Menu entry to set due date in one week.*/
		dueDateStore.append (out ti); 
        dueDateStore.set (ti, 0, _("In one week"), 1, 7);

        /* Menu entry to set due date in two weeks.*/
		dueDateStore.append (out ti); 
        dueDateStore.set (ti, 0, _("In two weeks"), 1, 14);

        /* Menu entry to reset due date.*/
		dueDateStore.append (out ti); 
        dueDateStore.set (ti, 0, _("No date"), 1, -2);

        /* Menu entry to chose a due date.*/
		dueDateStore.append (out ti); 
        dueDateStore.set (ti, 0, _("Choose Date..."), 1, -1);

		Utils.CellRendererCombo dueRenderer = new Utils.CellRendererCombo(
		    tree_view, dueDateStore, edit_area, Utils.UI.CellType.DUE_DATE);

		column.pack_start (dueRenderer, true);
		column.set_cell_data_func (dueRenderer, Utils.UI.dueDateCellDataFunc);

        return column;
    }

    public static Gtk.TreeViewColumn makeDeferColumn(Gtk.TreeView tree_view, Gtk.Bin edit_area){

		Gtk.TreeIter ti;
		Gtk.ListStore deferStore = new Gtk.ListStore (2, typeof(string), typeof(int));
        Gtk.TreeViewColumn column = new Gtk.TreeViewColumn ();

		// Title for Defer Column
		column.title = "Show";
		column.sizing = Gtk.TreeViewColumnSizing.AUTOSIZE;
		column.alignment = 0.5f;
		column.fixed_width = 20;
		column.resizable = false;
		column.clickable = true;

        deferStore.append (out ti); 
        /* Menu entry to cancel a defer action. */
        deferStore.set (ti, 0, _("Nevermind"), 1, 0); 
        deferStore.append (out ti); 

        /* Menu entry to defer for one day.*/
        deferStore.set (ti, 0, _("For one day"), 1, 1); 
        deferStore.append (out ti); 

        /* Menu entry to defer for one week.*/
        deferStore.set (ti, 0, _("For one week"), 1, 7);
        deferStore.append (out ti); 

        /* Menu entry to defer for two weeks.*/
        deferStore.set (ti, 0, _("For two weeks"), 1, 14);

        /* Menu entry to reset the defer date.*/
		deferStore.append (out ti); 
        deferStore.set (ti, 0, _("No date"), 1, -2);

        /* Menu entry to chose defer date.*/
        deferStore.append (out ti); 
        deferStore.set (ti, 0, _("Choose Date..."), 1, -1);

		Utils.CellRendererCombo deferRenderer = new Utils.CellRendererCombo(
		    tree_view, deferStore, edit_area, Utils.UI.CellType.DEFER);

		column.pack_start (deferRenderer, true);
		column.set_cell_data_func (deferRenderer, Utils.UI.showFromCellDataFunc);

        return column;
    }

    private static void showFromCellDataFunc (Gtk.CellLayout cell_layout,
				Gtk.CellRenderer cell,
				Gtk.TreeModel model,
				Gtk.TreeIter iter) {

		Utils.CellRendererCombo crc = cell as Utils.CellRendererCombo;
		ITask task;
        crc.tree_model.get (iter, 0, out task);
		bool showHeader = true;
        Gtk.TreeView tree = new Gtk.TreeView();
		string markup = Utils.UI.getDeferMarkup(task.showFrom, true, showHeader);
        if (markup != "") {
			crc.markup = markup; 
        } else {
            // No due date set
			crc.markup = "<span foreground=\"white\" background=\"%s\" weight=\"bold\"> ".printf(Utils.UI.getHighlightColor(tree)) + _("Defer") + " </span>";
        }
    }

    private static void dueDateCellDataFunc (Gtk.CellLayout cell_layout,
				Gtk.CellRenderer cell,
				Gtk.TreeModel model,
				Gtk.TreeIter iter) {

		Utils.CellRendererCombo crc = cell as Utils.CellRendererCombo;
		ITask task;
        crc.tree_model.get (iter, 0, out task);              
		if (task == null) return;
		bool showHeader = true;
        Gtk.TreeView tree = new Gtk.TreeView();
        if (task.dueAt.mktime() > 0) {
			crc.markup = Utils.UI.getDueMarkup(task.dueAt, true, showHeader);
        } else {
            /*Label displayed when no due date is set.*/   
	    	crc.markup = "<span foreground=\"white\" background=\"%s\" weight=\"bold\"> ".printf(Utils.UI.getHighlightColor(tree)) + _("Due") + " </span>";
        }
    }

	public static string getHighlightColor(Gtk.Widget widget) {
		Gdk.Color fgColor;
		Gtk.Style style = Gtk.rc_get_style(widget); 
		fgColor = style.bg [(int) Gtk.StateType.SELECTED];
		return fgColor.to_string();
	}

}

public class Utils.CellRendererCombo: Gtk.CellRendererCombo {

    private int calendarEvents;
    private ITask edited_task;
    private Gtk.Bin edit_area;
    private Gtk.TreeView _tree_view;
    private Logger log;

    public Gtk.TreeModel tree_model {
        get { return this._tree_view.model;}
    }
    
    public CellRendererCombo(Gtk.TreeView tree_view, Gtk.ListStore model, Gtk.Bin edit_area, Utils.UI.CellType type) {

 		this.editable = true;
		this.has_entry = false;
		this.text_column = 0;
		this.xalign = 0.5f;
        this.calendarEvents = 0;
        this.model = model;
        this._tree_view = tree_view;
        this.edited_task = null;
        this.edit_area = edit_area;

		if (type == Utils.UI.CellType.DEFER) {
       		this.edited += deferTask;
       	} else {
       	    this.edited += updateTaskDue;
       	}
       	
		this.log = Logger.getLogger("Utils.CellRendererCombo");

    }

    public void deferTask(Utils.CellRendererCombo sender, string path, string newValue ) {
        Gtk.TreeIter iter;
        ITask task;
        Gtk.TreePath aPath = new Gtk.TreePath.from_string(path);
        if (!sender.tree_model.get_iter(out iter, aPath))
				return; // Do nothing
        sender.tree_model.get(iter, 0, out task);	
        if ( task == null )
            return;	

        int days = -2;
        string name;
        int days_temp;        
        Gtk.TreeIter cIter;
        sender.model.get_iter_first(out cIter);
        do {
            sender.model.get (cIter, 0, out name, 1, out days_temp);
            if ( name == newValue ) {
                days = days_temp;
                break;
            }
        } while (sender.model.iter_next(ref cIter));        

        if ( days == 0 ) {
            return;
        }

        if ( days == -2 ) { // no date to set
           task.showFrom = GLib.Time();
           task.update();
           return;
        }

        if ( days > 0 ) {
           GLib.Date deferDate;
           GLib.Time deferTime;
           deferDate = Utils.DateTime.newDateNow();
           
           deferDate.add_days ( (uint)days );
           deferDate.to_time( out deferTime );
           task.showFrom = deferTime;
           task.update();
        }

        if ( days == -1 ) { // show calendar

            Gtk.Calendar calendar = new Gtk.Calendar();
            this.calendarEvents = 0;
            if (task.showFrom.mktime() > 0 ) { //select current date in calendar
                calendar.select_day ( task.showFrom.day );
                calendar.select_month ( task.showFrom.month,  task.showFrom.year + 1900);
            }
    
            calendar.focus_out_event += ( obj ) => {
                this.edit_area.remove(this.edit_area.get_child());
                this.edited_task = null;
                calendar = null;            
            };
            calendar.month_changed += ( obj ) => {  this.calendarEvents++;};
            calendar.day_selected += ( obj ) => {
                if ( this.calendarEvents == 0 ) {
                    uint year, month, day;
                    obj.get_date( out year, out month, out day );
                    GLib.Time calTime = GLib.Time();
                    calTime.strptime ("%u %u %u".printf(year, month + 1, day ),"%Y %m %d");
                    if (this.edited_task != null) {
                        this.edited_task.showFrom = calTime;
                        this.edited_task.update();      
                        this.edited_task = null;
                    }
                    this.edit_area.hide_all();
                    calendar = null;
                }
                this.calendarEvents = 0;
            };

            this.edit_area.add(calendar);
            this.edit_area.show_all();
            calendar.grab_focus();
            this.edited_task = task;
            return;
        }
    }

    public void updateTaskDue (Utils.CellRendererCombo sender, string path, string newValue ) {
        Gtk.TreeIter iter;
        ITask task;
        Gtk.TreePath aPath = new Gtk.TreePath.from_string( path );
        if (!sender.tree_model.get_iter (out iter, aPath))
				return; // Do nothing
		sender.tree_model.get (iter, 0, out task);	
        if ( task == null )
            return;	

        int days = -2;
        string name;
        int days_temp;        
        Gtk.TreeIter cIter;
        sender.model.get_iter_first(out cIter);
        do {
            sender.model.get (cIter, 0, out name, 1, out days_temp);
            if ( name == newValue ) {
                days = days_temp;
                break;
            }
        } while (sender.model.iter_next(ref cIter));        

        if ( days == -2 ) { // no date to set
           task.dueAt = GLib.Time();
           task.update();
           return;
        }
    
        if ( days == -1 ) { // show calendar

            Gtk.Calendar calendar = new Gtk.Calendar();
            this.calendarEvents = 0;
            if (task.dueAt.mktime() > 0 ) { //select current date in calendar
                //stdout.printf("TO cal %s\n",task.dueAt.to_string()); stdout.flush();  
                calendar.select_day ( task.dueAt.day );
                calendar.select_month ( task.dueAt.month,  task.dueAt.year + 1900);
            }
    
            calendar.focus_out_event += ( obj ) => {
                this.edit_area.remove(this.edit_area.get_child());
                this.edited_task = null;
                calendar = null;            
            };
            calendar.month_changed += ( obj ) => {  this.calendarEvents++;};
            calendar.day_selected += ( obj ) => {
                if ( calendarEvents == 0 ) {
                    uint year, month, day;
                    obj.get_date( out year, out month, out day );
                    
                    GLib.Time dueTime = GLib.Time();
                    dueTime.strptime ("%u %u %u".printf(year, month + 1, day ),"%Y %m %d");
                    //stdout.printf("FROM cal %u %u %u - conv %s\n",year, month, day,dueTime.to_string()); stdout.flush();  
                    if (this.edited_task != null) {
                        this.edited_task.dueAt = dueTime;
                        this.edited_task.update();      
                        this.edited_task = null;
                    }
                    this.edit_area.hide_all();
                    calendar = null;
                }
                this.calendarEvents = 0;
            };

            this.edit_area.add(calendar);
            this.edit_area.show_all();
            calendar.grab_focus();
            this.edited_task = task;
            return;
        }

        if ( days > -1 ) {
           GLib.Date dueDate;
           GLib.Time dueTime;
           dueDate = Utils.DateTime.newDateNow();
           dueDate.to_time( out dueTime );
           stdout.printf("FROM now %s\n",dueTime.to_string()); stdout.flush();
           dueDate.add_days ( (uint)days );
           dueDate.to_time( out dueTime );
           stdout.printf("After + %d now %s\n", days, dueTime.to_string()); stdout.flush();
           task.dueAt = dueTime;
           task.update();      
        }
    }

}

public class Utils.Entry : Gtk.Entry {

    string info;
    IView view;
    construct {
        info = "";
        view = null;
    }
    
    public Entry(string info){
        this.info = info;
        //this.view = view;

		this.set_width_chars(20);
		this.set_icon_from_stock(
		    Gtk.EntryIconPosition.PRIMARY, Gtk.STOCK_FIND);
        this.set_icon_activatable(
            Gtk.EntryIconPosition.PRIMARY, false);
		this.set_icon_from_stock(
		    Gtk.EntryIconPosition.SECONDARY, Gtk.STOCK_CLEAR);

        this.focus_in_event.connect(
            (entry, event) => {
                this.set_text(""); 
            });

        this.focus_out_event.connect(
            (entry, event) => {
                string current = this.get_text();
                if (current == "") {
                    this.set_text(this.info); 
                }
            });


//		this.icon_press.connect(
//		    (sender, position, event) => {
//		        this.set_text(this.info);
//		        this.view.search("");
//	        });

//		this.changed.connect(
//		    (sender) => {
//		        string text = this.get_text();
//		        if (text != "") {
//			        this.view.search(text);
//		        }
//	        });

        this.set_text(this.info);
    }
}
