using GLib;
using Gee;

public class Utils.DumbVala : Object {

    public static void dumbVala () {
       // this is a workaround to make Vala include config.h before
        // gi18n-lib.h
        string gettext_package = Config.GETTEXT_PACKAGE;
        Intl.bindtextdomain(gettext_package, Config.PACKAGE_LOCALEDIR);
        Intl.bind_textdomain_codeset(Config.GETTEXT_PACKAGE, "UTF-8");
        Intl.textdomain(Config.GETTEXT_PACKAGE);
    }

}


public class Utils.Constant: Object {
	public enum itemType { ITEM = 0, PROJECT, CATEGORY, NOTE, TASK}
	public const string filterNotes = _("Filter notes...");
	public const string filterTasks = _("Filter tasks...");
	public const string filterRecover = _("Filter tasks and notes...");
	public const string searchAll = _("Search tasks and notes...");
}

public class Utils.List : Object {

    public static void sort ( Gee.ArrayList<GLib.Object> list, Utils.Sort.CompareFunc func) {
        
        Utils.Sort.quickSort(list, 0, list.size-1, func);
    }


    public static void sortInt ( Gee.ArrayList<int> list, Utils.Sort.CompareFuncInt func) {
        
        Utils.Sort.quickSortInt(list, 0, list.size-1, func);
    }

}

public class Utils.Sort : Object {
    public delegate int CompareFunc ( Gee.ArrayList<GLib.Object> list, int left, int right);

    public delegate int CompareFuncInt ( Gee.ArrayList<int> list, int left, int right);


    public static void swapArrayList ( Gee.ArrayList<GLib.Object> list, int left, int right) {
        GLib.Object tempL = list.get(left);
        GLib.Object tempR = list.get(right);
        list.set(left,tempR);
        list.set(right,tempL);
    }

    public static void swapArrayListInt ( Gee.ArrayList<int> list, int left, int right) {
        int tempL = list.get(left);
        int tempR = list.get(right);
        list.set(left,tempR);
        list.set(right,tempL);
    }

    public static void quickSort(Gee.ArrayList<GLib.Object> list, int left, int right, Utils.Sort.CompareFunc func)
    {
        if (left >= right) return;
 
        int index = partition(list, left, right, func);
        Utils.Sort.quickSort(list, left, index - 1, func);
        Utils.Sort.quickSort(list, index + 1, right, func);
    }

    public static void quickSortInt(Gee.ArrayList<int> list, int left, int right, CompareFuncInt func)
    {

        if (left >= right) return;
 
        int index = partitionInt(list, left, right, func);
        Utils.Sort.quickSortInt(list, left, index - 1, func);
        Utils.Sort.quickSortInt(list, index + 1, right, func);
    }


    public static int partition(Gee.ArrayList<GLib.Object> list, int left, int right, Utils.Sort.CompareFunc func) {

        int pivotIndex = left;
        int comp = 1;
        int rightM = right;
        int leftM = left;
        while ( leftM < rightM ) {
            if ( comp == 1 ) {
                if ( func(list, rightM, pivotIndex) < 0) {
                    Utils.Sort.swapArrayList (list, pivotIndex, rightM);
                    pivotIndex = rightM;
                    leftM++;
                    comp = 0;
                } else { 
                    rightM--;
                }
            } else {
                if ( func(list, leftM, pivotIndex) > 0) {
                    Utils.Sort.swapArrayList (list, pivotIndex, leftM);
                    pivotIndex = leftM;
                    rightM--;
                    comp = 1;
                } else {
                    leftM++;
                }

            }
        }
        return pivotIndex;
    }


    public static int partitionInt(Gee.ArrayList<int> list, int left, int right, Utils.Sort.CompareFuncInt func) {

        int pivotIndex = left;
        int comp = 1;
        int rightM = right;
        int leftM = left;
        while ( leftM < rightM ) {
            if ( comp == 1 ) {
                if ( func(list, rightM, pivotIndex) < 0) {
                    Utils.Sort.swapArrayListInt (list, pivotIndex, rightM);
                    pivotIndex = rightM;
                    leftM++;
                    comp = 0;
                } else { 
                    rightM--;
                }
            } else {
                if ( func(list, leftM, pivotIndex) > 0) {
                    Utils.Sort.swapArrayListInt (list, pivotIndex, leftM);
                    pivotIndex = leftM;
                    rightM--;
                    comp = 1;
                } else {
                    leftM++;
                }

            }
        }
        return pivotIndex;
    }

}

public class Utils.DateTime : GLib.Object {
    /*
     date and time handling functions
    */
    public static GLib.Date newDateFromTime ( GLib.Time time ) {
        GLib.Date date = GLib.Date();
        date.clear();
        date.set_year( (GLib.DateYear)(time.year + 1900) );
        date.set_month( (GLib.DateMonth)(time.month + 1) );
        date.set_day( (GLib.DateDay)time.day );
        return date;
    }

    public static GLib.Date newDateNow () {
        GLib.Date date = GLib.Date();
        date.clear();
        time_t now = time_t ();
        GLib.Time timeNow = GLib.Time.gm( now );
        date.set_year( (GLib.DateYear)(timeNow.year + 1900) );
        date.set_month( (GLib.DateMonth)(timeNow.month + 1) );
        date.set_day( (GLib.DateDay)(timeNow.day));

        return date;
    }
	
    public static GLib.Time newTimeNow () {
        time_t now = time_t ();
        GLib.Time timeNow = GLib.Time.gm( now );
        return timeNow;
    }

	    public static GLib.Time parseTime (string text, string format) {
        GLib.Time tim = GLib.Time();
        tim.strptime (text, format);
        return tim;
    }

}
