public class ChestiiNote : GLib.Object , IItem, INote {
    /* properties */
    public int id { get; set; }
    public string name { get; set; }
    public string text { get; set; }
    public int projectID { get; set; }
    public GLib.Time updatedAt { get; set; }
    public GLib.Time createdAt { get; set; }
    public int width { get; set; }
    public int height { get; set; }
    public int x { get; set; }
    public int y { get; set; }
    public bool sticky { get; set; }
    public bool isDeleted { get; set; }

    public IBackend backend  { get; set;}
    public int instanceOf { get {return Utils.Constant.itemType.NOTE;}}

	/* signals */
    public signal void created ( bool status );
    public signal void deleted ( bool status );
    public signal void updated ( bool status );

    construct {
        name = "";
        text = "";
		projectID = 0;
        createdAt = GLib.Time ();
        updatedAt = GLib.Time ();
        height = 0;
        width = 0;
        x = 0;
        y = 0;
		sticky = false;
		isDeleted = false;
		backend = null;
    }

    public ChestiiNote (IBackend aBackend){
        this.backend = aBackend;
        this.projectID = 0;
	}

	public ChestiiNote.fromXML (IBackend aBackend, string xml){
		this.backend = aBackend;
		fillDataFromXML(xml);
	}

	public void push(){		
        (backend as ChestiiBackend).queuePostMessage ( "/index.php/notes/xml/add", this.toXML(),readAddResponse);
	}

    public void delete (){
        string uri = "/index.php/notes/xml/delete/%d".printf(this.id);
        (backend as ChestiiBackend).queueGetMessage ( uri, readDeleteResponse);
	}

    public void update (){
        string xml = this.toXML();
        string uri =  "/index.php/notes/xml/update/%d".printf(this.id);
        (backend as ChestiiBackend).queuePostMessage ( uri, xml, readUpdateResponse);
	}

    public string toXML() {
        string xml = "";
        xml += "<note>\n";
        xml += "\t<name><![CDATA[%s]]></name>\n".printf(this.name);
        xml += "\t<text><![CDATA[%s]]></text>\n".printf(this.text);
        xml += "\t<project-id>%d</project-id>\n".printf(this.projectID);
        xml += "\t<updated-at>%s</updated-at>\n".printf(this.updatedAt.format("%FT%T%z"));
        xml += "\t<created-at>%s</created-at>\n".printf(this.createdAt.format("%FT%T%z"));
        xml += "\t<width>%d</width>\n".printf(this.width);
        xml += "\t<height>%d</height>\n".printf(this.height);
        xml += "\t<x>%d</x>\n".printf(this.x);
        xml += "\t<y>%d</y>\n".printf(this.y);
        xml += "\t<sticky>%s</sticky>\n".printf((this.sticky) ? "1" : "0");
        xml += "</note>\n";
        return xml;
    }

/******************************************************************************
 * Private section
 */
	private void readAddResponse (Soup.Session session, Soup.Message message){

        bool status = (message.status_code == 201 ) ? true : false;
        stdout.printf("Cont|%u|: %s\n",message.status_code, message.response_body.data);
        stdout.flush();
		if (status) {
			fillDataFromXML(message.response_body.data);
		}
		this.created (status);
	}

	private void readDeleteResponse (Soup.Session session, Soup.Message message){
        bool status = (message.status_code == 201 ) ? true : false;
        stdout.printf("Cont: %s\n", message.response_body.data);
        stdout.flush();
		if (status) this.isDeleted = true;
        this.updated (status);
    }


   private void readUpdateResponse (Soup.Session session, Soup.Message message){
        bool status = (message.status_code == 201 ) ? true : false;
        stdout.printf("Cont: %s\n", message.response_body.data);
        stdout.flush();
        this.updated (status);
    }

    private string parserTextBuffer;
    private string parserCDataBuffer;

	private void fillDataFromXML (string xml) {
       	parserTextBuffer = "";
        GLib.MarkupParser markupParser = { noteParserStart,
                                           noteParserEnd,
                                           parserText, null, null };
        GLib.MarkupParseContext parser = new GLib.MarkupParseContext (markupParser, GLib.MarkupParseFlags.TREAT_CDATA_AS_TEXT, this, null);
        try {
            parser.parse(xml, xml.size());
            parser.end_parse();
        } catch (GLib.MarkupError ex) {
            stdout.printf("|%s|\n%s\n",xml, ex.message);
            return;
        }
	}

    private void parserText (GLib.MarkupParseContext context, string text, ulong text_len) throws GLib.MarkupError {
		parserCDataBuffer = parserTextBuffer;
        parserTextBuffer = text;
    }


    private void noteParserStart (GLib.MarkupParseContext context, string element_name, string[] attribute_names, string[] attribute_values) throws GLib.MarkupError {
		if ( element_name == "html") {
            throw new GLib.MarkupError.INVALID_CONTENT("Not a valid note XML");
        }

           //print ("start project name:" + element_name + "\n");
    }

    private void noteParserEnd (GLib.MarkupParseContext context, string element_name) throws GLib.MarkupError {
 
        if ( element_name == "name" ) {
            this.name = parserCDataBuffer.replace("]]>","");
        } else if ( element_name == "text" ) {
            this.text = parserCDataBuffer.replace("]]>","");
        } else if ( element_name == "id" ) {
            this.id = parserTextBuffer.to_int();
        } else if ( element_name == "project-id" ) {
            this.x = parserTextBuffer.to_int();
        } else if ( element_name == "x" ) {
            this.x = parserTextBuffer.to_int();
        } else if ( element_name == "y" ) {
            this.y = parserTextBuffer.to_int();
        } else if ( element_name == "width" ) {
            this.width = parserTextBuffer.to_int();
        } else if ( element_name == "height" ) {
            this.height = parserTextBuffer.to_int();
        } else if ( element_name == "created-at" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.createdAt = tim;
        } else if ( element_name == "updated-at" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.updatedAt = tim;
        } else {
            //print ("end " + element_name + "\n");
        }
    }

}
