public class ChestiiProject: GLib.Object, IItem, IProject {
    /* properties */
    public int id { get; set; }

    public GLib.Time completedAt { get; set; }
    public GLib.Time createdAt { get; set; }    
    public GLib.Time updatedAt { get; set; }
    public bool active { get; set; }
    public int position { get; set; }
    public string name { set; get; }
    public string text { set; get; }
    public bool isDeleted {get; set;}

    public IBackend backend  { get; set;}
    public int instanceOf { get {return Utils.Constant.itemType.PROJECT;}}

    /*signals*/
    public signal void created ( bool status );
    public signal void deleted ( bool status );
    public signal void updated ( bool status );

    construct {
        GLib.Rand rand = new GLib.Rand();
        id = (int)rand.next_int();
        name = _("Untitled Project");
        text = "Project description";
		position = 1000;
        updatedAt = GLib.Time();       
        createdAt = GLib.Time();
        completedAt = GLib.Time();
    }

	public ChestiiProject(IBackend aBackend) {
		backend = aBackend;
	}

	/* methods */

	public void push(){		
        (backend as ChestiiBackend).queuePostMessage ( "/index.php/projects/xml/add", this.toXML(),readAddResponse);
	}

    public void delete (){
        string uri = "/index.php/projects/xml/delete/%d".printf(this.id);
        (backend as ChestiiBackend).queueGetMessage ( uri, readDeleteResponse);
	}

    public void update (){
        string xml = this.toXML();
        string uri =  "/index.php/projects/xml/update/%d".printf(this.id);
        (backend as ChestiiBackend).queuePostMessage ( uri, xml, readUpdateResponse);
	}

    public string toXML() {
        string xml = "";
        xml += "<project>\n";
        xml += "\t<name><![CDATA[%s]]></name>\n".printf(this.name);
        xml += "\t<text><![CDATA[%s]]></text>\n".printf(this.text);
        xml += "\t<updated-at>%s</updated-at>\n".printf(this.updatedAt.format("%FT%T%z"));
        xml += "\t<created-at>%s</created-at>\n".printf(this.createdAt.format("%FT%T%z"));
        xml += "\t<completed-at>%s</completed-at>\n".printf(this.completedAt.format("%FT%T%z"));
        xml += "\t<position>%d</position>\n".printf(this.position);
        xml += "\t<active>%s</active>\n".printf((this.active ) ? "1": "0");
        xml += "</project>\n";
        return xml;
    }


/******************************************************************************
 * Private section
 */
	private void readAddResponse (Soup.Session session, Soup.Message message){

        bool status = (message.status_code == 201 ) ? true : false;
        stdout.printf("Cont add|%u|: %s\n",message.status_code, message.response_body.data);
        stdout.flush();
		if (status) {
			fillDataFromXML(message.response_body.data);
		}
		this.created (status);
	}

	private void readDeleteResponse (Soup.Session session, Soup.Message message){
        bool status = (message.status_code == 201 ) ? true : false;
        stdout.printf("Cont delete: %s\n", message.response_body.data);
        stdout.flush();
		if (status) this.isDeleted = true;
        this.updated (status);
    }


   private void readUpdateResponse (Soup.Session session, Soup.Message message){
        bool status = (message.status_code == 201 ) ? true : false;
        stdout.printf("Cont update: %s\n", message.response_body.data);
        stdout.flush();
        this.updated (status);
    }

    private string parserTextBuffer;
    private string parserCDataBuffer;

	private void fillDataFromXML (string xml) {
       	parserTextBuffer = "";
        GLib.MarkupParser markupParser = { projectParserStart,
                                           projectParserEnd,
                                           parserText, null, null };
        GLib.MarkupParseContext parser = new GLib.MarkupParseContext (markupParser, GLib.MarkupParseFlags.TREAT_CDATA_AS_TEXT, this, null);
        try {
            parser.parse(xml, xml.size());
            parser.end_parse();
        } catch (GLib.MarkupError ex) {
            stdout.printf("|%s|\n%s\n",xml, ex.message);
            return;
        }
	}

    private void parserText (GLib.MarkupParseContext context, string text, ulong text_len) throws GLib.MarkupError {
		parserCDataBuffer = parserTextBuffer;
        parserTextBuffer = text;
    }


    private void projectParserStart (GLib.MarkupParseContext context, string element_name, string[] attribute_names, string[] attribute_values) throws GLib.MarkupError {
		if ( element_name == "html") {
            throw new GLib.MarkupError.INVALID_CONTENT("Not a valid project XML");
        }

    }

    private void projectParserEnd (GLib.MarkupParseContext context, string element_name) throws GLib.MarkupError {
 
        if ( element_name == "name" ) {
            this.name = parserCDataBuffer.replace("]]>","");
        } else if ( element_name == "text" ) {
            this.text = parserCDataBuffer.replace("]]>","");
        } else if ( element_name == "id" ) {
            this.id = parserTextBuffer.to_int();
         } else if ( element_name == "position" ) {
            this.position = parserTextBuffer.to_int();
         } else if ( element_name == "active" ) {
            this.active = (bool) parserTextBuffer.to_int();
        } else if ( element_name == "created-at" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.createdAt = tim;
        } else if ( element_name == "updated-at" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.updatedAt = tim;
        } else if ( element_name == "completed-at" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.completedAt = tim;
		} else {
            //print ("end " + element_name + "\n");
		}
	}
}
