public class ChestiiTask : GLib.Object, IItem, ITask {
    public int id  { get; set; }
    public bool done  { get; set; }
    public int priority  { get; set; }
    public string name  { get; set; }
    public string text  { get; set; }

    public int categoryID  { get; set; }
    public int projectID  { get; set; }

    public GLib.Time createdAt { get; set; }
    public GLib.Time updatedAt { get; set; }
    public GLib.Time completedAt { get; set; }
    public GLib.Time dueAt { get; set; }
    public GLib.Time showFrom { get; set; }

    public bool isDeleted {get; set;}

	private ChestiiBackend _backend;
    public IBackend backend  { 
		get { return _backend;}
		set { _backend = (value as ChestiiBackend); }
	}

    public int instanceOf { get {return Utils.Constant.itemType.TASK;}}

    construct {
        _id = 0;
        _done = false;
        _name = _("Task Name");
        _priority = 0;
        _text = "";
        _projectID = 0;
        _categoryID = 0;
        _updatedAt = GLib.Time();
        _completedAt = GLib.Time();
        _showFrom = GLib.Time();
        _dueAt = GLib.Time();
        _createdAt = GLib.Time.gm(time_t ());
    }

    public ChestiiTask.empty () {
	}

    public ChestiiTask (IBackend aBackend) {
		backend = aBackend;
	}
  	    
	public ChestiiTask.fromXML( IBackend aBackend, string xml){
		backend = aBackend;

		//TODO implemente xml fill data
		//fillDataFromXML(xml);
	}

	public void push() {
        _backend.queuePostMessage ( "/index.php/tasks/xml/add", this.toXML(), readAddResponse);
	}

    public void delete () {
        string uri = "/index.php/tasks/xml/delete/%d".printf(this.id);
        _backend.queueGetMessage ( uri, readDeleteResponse);
    }
    
    public void update() {
        string xml = this.toXML();
        string uri = "/index.php/tasks/xml/update/%d".printf(this.id);
        stdout.printf("To web:\n %s\n",xml); stdout.flush();
        _backend.queuePostMessage ( uri, xml, readUpdateResponse);
    }
   
    public string toXML () {
        // FIXME send the correct timezone
        this.updatedAt = Utils.DateTime.newTimeNow();
        string xml = "<task>\n";
        xml += "\t<name><![CDATA[%s]]></name>\n".printf(this.name);
        xml += "\t<text><![CDATA[%s]]></text>\n".printf(this.text);
        xml += "\t<category-id>%d</category-id>\n".printf(this.categoryID);

        xml += "\t<project-id>%d</project-id>\n".printf(this.projectID);

        xml += "\t<done>%d</done>\n".printf((this.done ) ? 1 : 0);
        
        xml += "\t<updated-at>%s</updated-at>\n".printf(this.updatedAt.format("%FT%T%z"));

        xml += ( this.showFrom.mktime() > 0 ) ?
                "\t<show-from>%s</show-from>\n".printf(this.showFrom.format("%FT00:00:00%z")):
                "\t<show-from/>\n";
        
        xml += ( this.dueAt.mktime() > 0 ) ?
                "\t<due-at>%s</due-at>\n".printf(this.dueAt.format("%FT00:00:00%z")) :
                "\t<due-at/>\n";       

        xml += ( this.completedAt.mktime() > 0 ) ?
                "\t<completed-at>%s</completed-at>\n".printf(this.completedAt.format("%FT00:00:00%z")) :
                "";
     xml += "</task>\n";
        return xml;
    }

/******************************************************************************
 * Private section
 */
    private void readAddResponse (Soup.Session session, Soup.Message message){
        bool status = (message.status_code == 201 ) ? true : false;
        stdout.printf("Cont: %s\n", message.response_body.data); stdout.flush();
		if (status) fillDataFromXML(message.response_body.data);
        this.created (status);
    }

    private void readToggleResponse (Soup.Session session, Soup.Message message){
        bool status = (message.status_code == 201 ) ? true : false;
        stdout.printf("Cont: %s\n", message.response_body.data);
        stdout.flush();
		if (status) {
			fillDataFromXML(message.response_body.data);
			if ( this.done ) {
				this.completedAt = GLib.Time();
				this.done = false;
			} else {
				this.done = true;
				this.completedAt = GLib.Time.gm ( time_t ());
			}
		} 
        this.updated (status);
    }

   private void readUpdateResponse (Soup.Session session, Soup.Message message){
        bool status = (message.status_code == 201 ) ? true : false;
        stdout.printf("Cont: %s\n", message.response_body.data);
        stdout.flush();
        this.updated (status);
    }

   private void readDeleteResponse (Soup.Session session, Soup.Message message){
        bool status = (message.status_code == 201 ) ? true : false;
        stdout.printf("Cont: %s\n", message.response_body.data);
        stdout.flush();
		if (status) this.isDeleted = true;
		this.updated (status);
	}

    private string parserTextBuffer;
    private string parserCDataBuffer;

	private void fillDataFromXML (string xml) {
       	parserTextBuffer = "";
        GLib.MarkupParser markupParser = { taskParserStart,
                                           taskParserEnd,
                                           parserText, null, null };
        GLib.MarkupParseContext parser = new GLib.MarkupParseContext (markupParser, GLib.MarkupParseFlags.TREAT_CDATA_AS_TEXT, this, null);
        try {
            parser.parse(xml, xml.size());
            parser.end_parse();
        } catch (GLib.MarkupError ex) {
            stdout.printf("|%s|\n%s\n",xml, ex.message);
            return;
        }
	}

    private void parserText (GLib.MarkupParseContext context, string text, ulong text_len) throws GLib.MarkupError {
		parserCDataBuffer = parserTextBuffer;
        parserTextBuffer = text;
    }

    private void taskParserStart (GLib.MarkupParseContext context, string element_name, string[] attribute_names, string[] attribute_values) throws GLib.MarkupError {
           //print ("start project name:" + element_name + "\n");
    }

    private void taskParserEnd (GLib.MarkupParseContext context, string element_name) throws GLib.MarkupError {

		if ( element_name == "done" ) {
            this.done = (bool)parserTextBuffer.to_int();
        } else if ( element_name == "name" ) {
            this.name = parserCDataBuffer.replace("]]>","");
        } else if ( element_name == "text" ) {
            this.text = parserCDataBuffer.replace("]]>","");
        } else if ( element_name == "id" ) {
            this.id = parserTextBuffer.to_int();
        } else if ( element_name == "project-id" ) {
            this.projectID = parserTextBuffer.to_int();
        } else if ( element_name == "context-id" ) {
            this.categoryID = parserTextBuffer.to_int();
        } else if ( element_name == "priority" ) {
            this.priority = parserTextBuffer.to_int();
		} else if ( element_name == "created-at" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.createdAt = tim;
		} else if ( element_name == "updated-at" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.updatedAt = tim;
        } else if ( element_name == "completed-at" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.completedAt = tim;
        } else if ( element_name == "due-at" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.dueAt = tim;
        } else if ( element_name == "show-from" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.showFrom = tim;
       	}
   }

}
