public class SQLiteCategory: GLib.Object, IItem, ICategory {
    /* properties */
    public int id { get; set; }

    public GLib.Time createdAt { get; set; }    
    public GLib.Time updatedAt { get; set; }
    public bool hide { get; set; }
    public int position { get; set; }
    public string name { set; get; }
    public string text { get; set; }
    public bool isDeleted {get; set;}

    public IBackend backend  { get; set;}
    public int instanceOf { get {return Utils.Constant.itemType.CATEGORY;}}

    /*signals*/
    public signal void created ( bool status );
    public signal void deleted ( bool status );
    public signal void updated ( bool status );

	private Logger log;

    construct {
        GLib.Rand rand = new GLib.Rand();
        id = (int)rand.next_int();
        name = _("Untitled Category");
        hide = false;
		text = "";
        createdAt = Utils.DateTime.newTimeNow();
		updatedAt = Utils.DateTime.newTimeNow();
		log = Logger.getLogger("SQLiteCategory");
    }
	public SQLiteCategory(IBackend aBackend) {
		backend = aBackend;
	}


	public SQLiteCategory.fromArray (IBackend aBackend, string[] rows, HashTable<string,int> index) {
		backend = aBackend;
		fillDataFromArray(rows, index);
	}
	/* methods */

	public void push(){
		GLib.Idle.add(idlePush);
	}

    public void delete (){
		GLib.Idle.add(idleDelete);
	}

    public void update (){
		GLib.Idle.add(idleUpdate);
	}


/******************************************************************************
 * PRIVATE
 */

    private bool idlePush(){
		string query = "INSERT INTO categories ";
		query += "(name, updated_at, created_at, position, hide )";
		query += " VALUES ";
		query += "('%s','%s','%s','%d','%d')". printf(
					this.name.replace("'","''"),
					this.updatedAt.format("%Y-%m-%d %H:%M:%S"),
					this.createdAt.format("%Y-%m-%d %H:%M:%S"),
					this.position,
					(int)this.hide
					);
		query += ";";
		int rc = (backend as SQLiteBackend).db.exec(query);
		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			this.created(false);
		} else {
			this.id = (int)(backend as SQLiteBackend).db.last_insert_rowid ();
			log.debug("New category push with id %d".printf(this.id));
			this.created(true);
		}
		return false;
	}

    private bool idleDelete (){
		string query = "DELETE FROM categories WHERE id = '%d';".printf(this.id);
		int rc = (backend as SQLiteBackend).db.exec(query);
		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			this.updated(false);
		} else {
			this.isDeleted = true;
			this.updated(true);
		}
		return false;
	}

    private bool idleUpdate (){
		this.updatedAt = Utils.DateTime.newTimeNow();
		string query = "UPDATE categories SET ";
		query += "name = '%s' , ".printf(this.name.replace("'","''"));
		query += "position = '%d' , ".printf(this.position);
		query += "updated_at = '%s' , ".printf(this.updatedAt.format("%Y-%m-%d %H:%M:%S"));
		query += "created_at = '%s' , ".printf(this.createdAt.format("%Y-%m-%d %H:%M:%S"));
		query += "hide = '%d' ".printf((int)this.hide);
		query += " WHERE id='%d';".printf(this.id);
		int rc = (backend as SQLiteBackend).db.exec(query);


		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			this.updated(false);
		} else {
			this.updated(true);
		}
		return false;
	}


	private void fillDataFromArray( string[] result, HashTable<string,int> columnIndex) {
		int? index;

		index = columnIndex.lookup("id");
		if (index != null) {
			string text = result[0+columnIndex.lookup("id")];
			if (text !=null) this.id = text.to_int(); 
		}

		index = columnIndex.lookup("name");
		if (index != null) {
			string text = result[0+columnIndex.lookup("name")];
			if (text != null) this.name = text; 
		}

		index = columnIndex.lookup("position");
		if (index != null) {
			string text = result[0+columnIndex.lookup("position")];
			if (text != null) this.position = text.to_int();
		}

		index = columnIndex.lookup("hide");
		if (index != null) {
		string text = result[0+columnIndex.lookup("hide")];
		if (text != null) this.hide = (bool)text.to_int();
		}

		index = columnIndex.lookup("updated_at");
		if (index != null) {
			string text = result[0+columnIndex.lookup("updated_at")];
			if (text != null) this.updatedAt = Utils.DateTime.parseTime(text, "%Y-%m-%dT%T");
		}

		index = columnIndex.lookup("created_at");
		if (index != null) {
			string text = result[0+columnIndex.lookup("created_at")];
			if (text != null) this.createdAt = Utils.DateTime.parseTime(text,	"%Y-%m-%dT%T");
		}

	}

}
