using Sqlite;

public class SQLiteNote : GLib.Object , IItem, INote {
    /* properties */
    public int id { get; set; }
    public string name { get; set; }
    public string text { get; set; }
    public int projectID { get; set; }
    public GLib.Time updatedAt { get; set; }
    public GLib.Time createdAt { get; set; }
    public int width { get; set; }
    public int height { get; set; }
    public int x { get; set; }
    public int y { get; set; }
    public bool sticky { get; set; }

    public bool isDeleted { get; set; }
    public IBackend backend  { get; set;}
    public int instanceOf { get {return Utils.Constant.itemType.NOTE;}}

	/* signals */
    public signal void created ( bool status );
    public signal void deleted ( bool status );
    public signal void updated ( bool status );

	private Logger log;

    construct {
        name = "Untitled Note";
        text = "";
		projectID = 0;
        createdAt = Utils.DateTime.newTimeNow();
        updatedAt = Utils.DateTime.newTimeNow();
        height = 350;
        width = 450;
        x = 0;
        y = 0;
		sticky = false;
		isDeleted = false;
		backend = null;
		log = Logger.getLogger("SQLiteNote");
    }

    public SQLiteNote (IBackend aBackend){
        this.backend = aBackend;
	}

	public SQLiteNote.fromArray (IBackend aBackend, string[] rows, HashTable<string,int> index) {
		this.backend = aBackend;
		fillDataFromArray(rows, index);
	}

	public void push(){
		GLib.Idle.add(idlePush);
	}

    public void delete (){
		GLib.Idle.add(idleDelete);
	}

    public void update (){
		GLib.Idle.add(idleUpdate);
	}


/******************************************************************************
 * PRIVATE
 */


	private bool idlePush(){
		log.debug("Start push");
		string query = "INSERT INTO notes ";
		query += "(name, text, project_id, updated_at, created_at, width, height, x, y, sticky )";
		query += " VALUES ";
		query += "('%s','%s','%d','%s','%s','%d','%d','%d','%d','%d')". printf(
					this.name.replace("'","''"),
					this.text.replace("'","''"),
					this.projectID,
					this.updatedAt.format("%Y-%m-%d %H:%M:%S"),
					this.createdAt.format("%Y-%m-%d %H:%M:%S"),
					this.width,
					this.height,
					this.x,
					this.y,
					(int)this.sticky
					);
		query += ";";
		int rc = (backend as SQLiteBackend).db.exec(query);
		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			this.created(false);
		} else {
			this.id = (int)(backend as SQLiteBackend).db.last_insert_rowid ();
			this.created(true);
			this.backend.updateNotes();
		}
		log.debug("End push");
		return false;
	}

    private bool idleDelete (){
		log.debug("Start delete");
		string query = "DELETE FROM notes WHERE id = '%d';".printf(this.id);
		int rc = (backend as SQLiteBackend).db.exec(query);
		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			this.updated(false);
		} else {
			this.isDeleted = true;
			this.updated(true);
		}
		log.debug("End delete");
		return false;
	}

    private bool idleUpdate (){
		log.debug("Start update");
		this.updatedAt = Utils.DateTime.newTimeNow();
		string query = "UPDATE notes SET ";
		query += "name = '%s' , ".printf(this.name.replace("'","''"));
		query += "text = '%s' , ".printf(this.text.replace("'","''"));
		query += "project_id = '%d' , ".printf(this.projectID);
		query += "width = '%d' , ".printf(this.width);
		query += "height = '%d' , ".printf(this.height);
		query += "x = '%d' , ".printf(this.x);
		query += "y = '%d' , ".printf(this.y);
		query += "updated_at = '%s' , ".printf(this.updatedAt.format("%Y-%m-%d %H:%M:%S"));
		query += "created_at = '%s' , ".printf(this.createdAt.format("%Y-%m-%d %H:%M:%S"));
		query += "sticky = '%d' ".printf((int)this.sticky);
		query += " WHERE id='%d';".printf(this.id);
		int rc = (backend as SQLiteBackend).db.exec(query);


		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			//this.updated(false);
		} else {
			//this.updated(true);
		}
		log.debug("End update");
		return false;
	}


	private void fillDataFromArray( string[] result, HashTable<string,int> columnIndex) {
		weak int? index;

		index = columnIndex.lookup("id");
		if (index != null) {
			string text = result[0+columnIndex.lookup("id")];
			if (text !=null) this.id = text.to_int(); 
		}

		index = columnIndex.lookup("name");
		if (index != null) {
			string text = result[0+columnIndex.lookup("name")];
			if (text != null) this.name = text; 
		}

		index = columnIndex.lookup("text");
		if (index != null) {
			string text = result[0+columnIndex.lookup("text")];
			if (text != null) this.text = text;
		}

		index = columnIndex.lookup("width");
		if (index != null) {
			string text = result[0+columnIndex.lookup("width")];
			if (text != null) this.width = text.to_int();
		}


		index = columnIndex.lookup("height");
		if (index != null) {
			string text = result[0+columnIndex.lookup("height")];
			if (text != null) this.height = text.to_int();
		}

		index = columnIndex.lookup("x");
		if (index != null) {
			string text = result[0+columnIndex.lookup("x")];
			if (text != null) this.x = text.to_int();
		}

		index = columnIndex.lookup("y");
		if (index != null) {
			string text = result[0+columnIndex.lookup("y")];
			if (text != null) this.y = text.to_int();
		}
		index = columnIndex.lookup("sticky");
		if (index != null) {
		string text = result[0+columnIndex.lookup("sticky")];
		if (text != null) this.sticky = (bool)text.to_int();
		}

		index = columnIndex.lookup("project_id");
		if (index != null) {
			string text = result[0+columnIndex.lookup("project_id")];
			if (text != null) this.projectID = text.to_int();
		}

		index = columnIndex.lookup("updated_at");
		if (index != null) {
			string text = result[0+columnIndex.lookup("updated_at")];
			if (text != null) this.updatedAt = Utils.DateTime.parseTime(text, "%Y-%m-%dT%T");
		}

		index = columnIndex.lookup("created_at");
		if (index != null) {
			string text = result[0+columnIndex.lookup("created_at")];
			if (text != null) this.createdAt = Utils.DateTime.parseTime(text,	"%Y-%m-%dT%T");
		}

	}
}
