public class SQLiteProject: GLib.Object, IItem, IProject {
    /* properties */
    public int id { get; set; }

    public GLib.Time completedAt { get; set; }
    public GLib.Time createdAt { get; set; }    
    public GLib.Time updatedAt { get; set; }
    public bool active { get; set; }
    public int position { get; set; }
    public string name { set; get; }
    public string text { set; get; }

    public bool isDeleted {get; set;}
    public IBackend backend  { get; set;}
    public int instanceOf { get {return Utils.Constant.itemType.PROJECT;}}

    /*signals*/
    public signal void created ( bool status );
    public signal void deleted ( bool status );
    public signal void updated ( bool status );

	private Logger log;

    construct {
        GLib.Rand rand = new GLib.Rand();
        id = (int)rand.next_int();
        name = _("Untitled Project");
        text = "Project description";
		position = 1000;
        updatedAt = Utils.DateTime.newTimeNow();       
        createdAt = Utils.DateTime.newTimeNow();
        completedAt = GLib.Time();
		log = Logger.getLogger("SQLiteProject");

    }

	public SQLiteProject(IBackend aBackend) {
		backend = aBackend;
	}

	public SQLiteProject.fromArray (IBackend aBackend, string[] rows, HashTable<string,int> index) {
		backend = aBackend;
		fillDataFromArray(rows, index);
	}

	/* methods */

	public void push(){
		GLib.Idle.add(idlePush);
	}

    public void delete (){
		GLib.Idle.add(idleDelete);
	}

    public void update (){
		GLib.Idle.add(idleUpdate);
	}

/******************************************************************************
 * PRIVATE
 */

    private bool idlePush(){
		string query = "INSERT INTO projects ";
		query += "(name, text, updated_at, created_at, position, active )";
		query += " VALUES ";
		query += "('%s','%s','%s','%s','%d','%d')". printf(
					this.name.replace("'","''"),
					this.text.replace("'","''"),
					this.updatedAt.format("%Y-%m-%d %H:%M:%S"),
					this.createdAt.format("%Y-%m-%d %H:%M:%S"),
					this.position,
					(int)this.active
					);
		query += ";";
		int rc = (backend as SQLiteBackend).db.exec(query);
		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			this.created(false);
		} else {
			this.id = (int)(backend as SQLiteBackend).db.last_insert_rowid ();
			this.created(true);
		}
		return false;
	}

    private bool idleDelete (){
		string query = "DELETE FROM projects WHERE id = '%d';".printf(this.id);
		int rc = (backend as SQLiteBackend).db.exec(query);
		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			this.updated(false);
		} else {
			this.isDeleted = true;
			this.updated(true);
		}
		return false;
	}

    private bool idleUpdate (){
		this.updatedAt = Utils.DateTime.newTimeNow();
		string query = "UPDATE projects SET ";
		query += "name = '%s' , ".printf(this.name.replace("'","''"));
		query += "text = '%s' , ".printf(this.text.replace("'","''"));
		query += "position = '%d' , ".printf(this.position);
		query += "updated_at = '%s' , ".printf(this.updatedAt.format("%Y-%m-%d %H:%M:%S"));
		query += "created_at = '%s' , ".printf(this.createdAt.format("%Y-%m-%d %H:%M:%S"));
		query += "active = '%d' ".printf((int)this.active);
		query += " WHERE id='%d';".printf(this.id);
		int rc = (backend as SQLiteBackend).db.exec(query);


		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			this.updated(false);
		} else {
			this.updated(true);
		}
		return false;
	}


	private void fillDataFromArray( string[] result, HashTable<string,int> columnIndex) {
		int? index;

		index = columnIndex.lookup("id");
		if (index != null) {
			string text = result[0+columnIndex.lookup("id")];
			if (text !=null) this.id = text.to_int(); 
		}

		index = columnIndex.lookup("name");
		if (index != null) {
			string text = result[0+columnIndex.lookup("name")];
			if (text != null) this.name = text; 
		}

		index = columnIndex.lookup("text");
		if (index != null) {
			string text = result[0+columnIndex.lookup("text")];
			if (text != null) this.text = text;
		}

		index = columnIndex.lookup("position");
		if (index != null) {
			string text = result[0+columnIndex.lookup("position")];
			if (text != null) this.position = text.to_int();
		}

		index = columnIndex.lookup("active");
		if (index != null) {
		string text = result[0+columnIndex.lookup("active")];
		if (text != null) this.active = (bool)text.to_int();
		}

		index = columnIndex.lookup("updated_at");
		if (index != null) {
			string text = result[0+columnIndex.lookup("updated_at")];
			if (text != null) this.updatedAt = Utils.DateTime.parseTime(text, "%Y-%m-%dT%T");
		}

		index = columnIndex.lookup("created_at");
		if (index != null) {
			string text = result[0+columnIndex.lookup("created_at")];
			if (text != null) this.createdAt = Utils.DateTime.parseTime(text,	"%Y-%m-%dT%T");
		}

		index = columnIndex.lookup("completed_at");
		if (index != null) {
			string text = result[0+columnIndex.lookup("completed_at")];
			if (text != null) this.completedAt = Utils.DateTime.parseTime(text, "%Y-%m-%dT%T");
		}

	}
}
