public class SQLiteTask : GLib.Object, IItem, ITask {
    public int id  { get; set; }
    public bool done  { get; set; }
    public int priority  { get; set; }
    public string name  { get; set; }
    public string text  { get; set; }

    public int categoryID  { get; set; }
    public int projectID  { get; set; }

    public GLib.Time createdAt { get; set; }
    public GLib.Time updatedAt { get; set; }
    public GLib.Time completedAt { get; set; }
    public GLib.Time dueAt { get; set; }
    public GLib.Time showFrom { get; set; }

    public bool isDeleted {get; set;}

	private SQLiteBackend _backend;
    public IBackend backend  { 
		get { return _backend;}
		set { _backend = (value as SQLiteBackend); }
	}

    public int instanceOf { get {return Utils.Constant.itemType.TASK;}}

	private Logger log;

    construct {
        _id = 0;
        _done = false;
        _name = _("Task Name");
        _priority = 0;
        _text = "";
        _projectID = 0;
        _categoryID = 0;
        _updatedAt = Utils.DateTime.newTimeNow();
        _completedAt = GLib.Time();
        _showFrom = GLib.Time();
        _dueAt = GLib.Time();
        _createdAt = Utils.DateTime.newTimeNow();
		log = Logger.getLogger("SQLiteTask");
    }

    public SQLiteTask.empty () {
	}

    public SQLiteTask (IBackend aBackend) {
		backend = aBackend;
	}

	public SQLiteTask.fromArray (IBackend aBackend, string[] rows, HashTable<string,int> index) {
		this.backend = aBackend;
		fillDataFromArray(rows, index);
	}

	public void push(){
		GLib.Idle.add(idlePush);
	}

    public void delete (){
		GLib.Idle.add(idleDelete);
	}

    public void update (){
		GLib.Idle.add(idleUpdate);
	}

   
/******************************************************************************
 * PRIVATE
 */

	private bool idlePush () {
		log.debug("Start push");
		string query = "INSERT INTO tasks ";
		query += "(name, text, project_id, category_id, updated_at, created_at, completed_at, due_at, show_from , done )";
		query += " VALUES ";
		query += "('%s','%s','%d','%d','%s','%s','%s','%s','%s','%d')". printf(
					this.name,
					this.text,
					this.projectID,
					this.categoryID,
					this.updatedAt.format("%Y-%m-%d %H:%M:%S"),
					this.createdAt.format("%Y-%m-%d %H:%M:%S"),
					this.completedAt.format("%Y-%m-%d %H:%M:%S"),
					this.dueAt.format("%Y-%m-%d %H:%M:%S"),
					this.showFrom.format("%Y-%m-%d %H:%M:%S"),
					(int)this.done
					);
		query += ";";
		int rc = (backend as SQLiteBackend).db.exec(query);
		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			this.created(false);
		} else {
			this.id = (int)(backend as SQLiteBackend).db.last_insert_rowid ();
			this.created(true);
			this.backend.updateTasks();
		}
		log.debug("End push");
		return false;
	}	

	private bool idleUpdate () {
		log.debug("Start update");
		this.updatedAt = Utils.DateTime.newTimeNow();
		string query = "UPDATE tasks SET ";
		query += "name = '%s' , ".printf(this.name.replace("'","''"));
		query += "text = '%s' , ".printf(this.text.replace("'","''"));
		query += "project_id = '%d' , ".printf(this.projectID);
		query += "category_id = '%d' , ".printf(this.categoryID);
		query += "updated_at = '%s' , ".printf(this.updatedAt.format("%Y-%m-%d %H:%M:%S"));
		query += "created_at = '%s' , ".printf(this.createdAt.format("%Y-%m-%d %H:%M:%S"));
		query += "completed_at = '%s' , ".printf(this.completedAt.format("%Y-%m-%d %H:%M:%S"));
		query += "due_at = '%s' , ".printf(this.dueAt.format("%Y-%m-%d %H:%M:%S"));
		query += "show_from = '%s' , ".printf(this.showFrom.format("%Y-%m-%d %H:%M:%S"));
		query += "done = '%d' ".printf((int)this.done);
		query += " WHERE id='%d';".printf(this.id);
		int rc = (backend as SQLiteBackend).db.exec(query);

		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			this.updated(false);
		} else {
			this.updated(true);
		}
		log.debug("End update");
		return false;
	}	


	private bool idleDelete () {
		log.debug("Start delete");
		string query = "DELETE FROM tasks WHERE id = '%d';".printf(this.id);
		int rc = (backend as SQLiteBackend).db.exec(query);
		if (rc != Sqlite.OK) {
			log.debug("DB error: " +  (backend as SQLiteBackend).db.errmsg ());
			this.updated(false);
		} else {
			this.isDeleted = true;
			this.updated(true);
		}
		log.debug("End delete");
		return false;
	}

	private void fillDataFromArray( string[] result, HashTable<string,int> columnIndex) {
		weak int? index;

		index = columnIndex.lookup("id");
		if (index != null) {
			string text = result[0+columnIndex.lookup("id")];
			if (text !=null) this.id = text.to_int(); 
		}

		index = columnIndex.lookup("name");
		if (index != null) {
			string text = result[0+columnIndex.lookup("name")];
			if (text != null) this.name = text; 
		}

		index = columnIndex.lookup("text");
		if (index != null) {
			string text = result[0+columnIndex.lookup("text")];
			if (text != null) this.text = text;
		}

		index = columnIndex.lookup("prority");
		if (index != null) {
			string text = result[0+columnIndex.lookup("priority")];
			if (text != null) this.priority = text.to_int();
		}

		index = columnIndex.lookup("done");
		if (index != null) {
		string text = result[0+columnIndex.lookup("done")];
		if (text != null) this.done = (bool)text.to_int();
		}

		index = columnIndex.lookup("project_id");
		if (index != null) {
			string text = result[0+columnIndex.lookup("project_id")];
			if (text != null) this.projectID = text.to_int();
		}

		index = columnIndex.lookup("category_id");
		if (index != null) {
			string text = result[0+columnIndex.lookup("category_id")];
			if (text != null) this.categoryID = text.to_int();
		}


		index = columnIndex.lookup("updated_at");
		if (index != null) {
			string text = result[0+columnIndex.lookup("updated_at")];
			if (text != null) this.updatedAt = Utils.DateTime.parseTime(text, "%Y-%m-%dT%T");
		}

		index = columnIndex.lookup("created_at");
		if (index != null) {
			string text = result[0+columnIndex.lookup("created_at")];
			if (text != null) this.createdAt = Utils.DateTime.parseTime(text,	"%Y-%m-%dT%T");
		}

		index = columnIndex.lookup("completed_at");
		if (index != null) {
			string text = result[0+columnIndex.lookup("completed_at")];
			if (text != null) this.completedAt = Utils.DateTime.parseTime(text, "%Y-%m-%dT%T");
		}

		index = columnIndex.lookup("due_at");
		if (index != null) {
			string text = result[0+columnIndex.lookup("due_at")];
			if (text != null) this.dueAt = Utils.DateTime.parseTime(text, "%Y-%m-%dT%T");
		}

		index = columnIndex.lookup("show_from");
		if (index != null) {
			string text = result[0+columnIndex.lookup("show_from")];
			if (text != null) this.showFrom = Utils.DateTime.parseTime(text, "%Y-%m-%dT%T");
		}


	}
}
