public class TomboyNote : GLib.Object , INote {
    /* properties */
    public string name { get; set; }
    public string text { get; set; }
    public GLib.Time updatedAt { get; set; }
    public GLib.Time createdAt { get; set; }
    public int cursorPosition { get; set; }
    public int width { get; set; }
    public int height { get; set; }
    public int x { get; set; }
    public int y { get; set; }

    private string parserTextBuffer;

    construct {
        name = "";
        text = "";
        createdAt = GLib.Time ();
        updatedAt = GLib.Time ();
        cursorPosition = 0;
        height = 0;
        width = 0;
        x = 0;
        y = 0;
    }

    public TomboyNote.fromXML ( string xml ) {

        parserTextBuffer = "";
        GLib.MarkupParser markupParser = { noteParserStart,
                                           noteParserEnd,
                                           parserText, null, null };
        GLib.MarkupParseContext parser = new GLib.MarkupParseContext (markupParser, GLib.MarkupParseFlags.TREAT_CDATA_AS_TEXT, this, null);
        try {
            parser.parse(xml, xml.size());
            parser.end_parse();
        } catch (GLib.MarkupError ex) {
            stdout.printf("|%s|\n%s\n",xml, ex.message);
            return;
        }

    }


    public string toXML() {
        string xml = "";
        xml += "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
        xml += "<note version=\"0.3\" xmlns:link=\"http://beatniksoftware.com/tomboy/link\" xmlns:size=\"http://beatniksoftware.com/tomboy/size\" xmlns=\"http://beatniksoftware.com/tomboy\">\n";
        xml += "\t<title>%s</title>\n".printf(this.name);
        xml += "\t<text>%s</text>\n".printf(this.text);
        xml += "\t<last-change-date>%s</last-change-date>\n".printf(this.updatedAt.format("%FT%T%z"));
        xml += "\t<last-metadata-change-date>%s</last-change-date>\n".printf(this.updatedAt.format("%FT%T%z"));
        xml += "\t<created-date>%s</last-change-date>\n".printf(this.createdAt.format("%FT%T%z"));
        xml += "\t<cursor-position>%d</cursor-position>\n".printf(this.cursorPosition);
        xml += "\t<width>%d</width>\n".printf(this.width);
        xml += "\t<height>%d</height>\n".printf(this.height);
        xml += "\t<x>%d</x>\n".printf(this.x);
        xml += "\t<y>%d</y>\n".printf(this.y);
        xml += "\t<open-on-startup>False</open-on-startup>\n";
        xml += "</note>\n";

        return xml;
    }

    private void parserText (GLib.MarkupParseContext context, string text, ulong text_len) throws GLib.MarkupError {
        //stdout.printf ("text %ld|%s|\n",(long)text_len,text);
        parserTextBuffer = text;
    }


    private void noteParserStart (GLib.MarkupParseContext context, string element_name, string[] attribute_names, string[] attribute_values) throws GLib.MarkupError {
           //print ("start project name:" + element_name + "\n");
    }

    private void noteParserEnd (GLib.MarkupParseContext context, string element_name) throws GLib.MarkupError {
 
        if ( element_name == "title" ) {
            this.name = parserTextBuffer;
        } else if ( element_name == "note-content" ) {
            this.text = parserTextBuffer;
        } else if ( element_name == "x" ) {
            this.x = parserTextBuffer.to_int();
        } else if ( element_name == "y" ) {
            this.y = parserTextBuffer.to_int();
        } else if ( element_name == "width" ) {
            this.width = parserTextBuffer.to_int();
        } else if ( element_name == "height" ) {
            this.height = parserTextBuffer.to_int();
        } else if ( element_name == "cursor-position" ) {
            this.cursorPosition = parserTextBuffer.to_int();
        } else if ( element_name == "created-date" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.createdAt = tim;
        } else if ( element_name == "last-change-date" ) {
            GLib.Time tim = GLib.Time();
            tim.strptime (parserTextBuffer, "%Y-%m-%dT%T");
            this.updatedAt = tim;
        } else {
            //print ("end " + element_name + "\n");
        }
    }

}
