public class TracksTask : GLib.Object, ITask {
    public int id  { get; set; }
    public bool done  { get; set; }
    public int priority  { get; set; }
    public string name  { get; set; }
    public string text  { get; set; }

    public int categoryID  { get; set; }
    public int projectID  { get; set; }

    public GLib.Time createdAt { get; set; }
    public GLib.Time updatedAt { get; set; }
    public GLib.Time completedAt { get; set; }
    public GLib.Time dueAt { get; set; }
    public GLib.Time showFrom { get; set; }

    public ITasksBackend backend  { get; set; }

    construct {
        _id = 0;
        _done = false;
        _name = "Task name";
        _priority = 0;
        _text = "";

        _projectID = 0;
        _categoryID = 0;


        _updatedAt = GLib.Time();
        _completedAt = GLib.Time();
        _showFrom = GLib.Time();
        _dueAt = GLib.Time();
        _createdAt = GLib.Time.gm(time_t ());
    }

    //private TracksTask () {
        
    //}
  
    public TracksTask.withBackend ( ITasksBackend aBackend ) {
        _backend = aBackend;
    }
    
    public void toggle() {
        if ( this.done ) {
            this.completedAt = GLib.Time();
            this.done = false;
        } else {
            this.done = true;
            this.completedAt = GLib.Time.gm ( time_t ());
        }
        
    }
    
    public string to_xml () {
        /*
        <todo>
            <completed-at type="datetime" nil="true"/>
            <context-id type="integer">3</context-id>
            <created-at type="datetime">2009-03-03T13:19:21+02:00</created-at>
            <description>contract comodat</description>
            <due type="datetime">2009-03-04T02:00:00+02:00</due>
            <id type="integer">1</id>
            <notes>desciere contract comodat....stared, cu tagur</notes>
            <project-id type="integer">1</project-id>
            <recurring-todo-id type="integer" nil="true"/>
            <show-from type="datetime" nil="true"/>
            <state>active</state>
            <updated-at type="datetime">2009-03-05T15:59:22+02:00</updated-at>
        </todo>
        */
        // TODO send the correct timezone
        this.updatedAt = Utils.DateTime.newTimeNow();
        string xml = "<todo>\n";
        xml += "\t<description>%s</description>\n".printf(this.name);

        xml += "\t<context-id type=\"integer\">%d</context-id>\n".printf(this.categoryID);
         
        xml += (this.done ) ? "\t<state>completed</state>\n" : "\t<state>active</state>\n";
        
        xml += "\t<updated-at type=\"datetime\">%s</updated-at>\n".printf(this.updatedAt.format("%FT%T%z"));

        xml += ( this.showFrom.mktime() > 0 ) ?
                "\t<show-from type=\"datetime\">%s</show-from>\n".printf(this.showFrom.format("%FT00:00:00%z")) :
                "";
        
        xml += ( this.dueAt.mktime() > 0 ) ?
                "\t<due type=\"datetime\" >%s</due>\n".printf(this.dueAt.format("%FT00:00:00%z")) :
                "\t<due type=\"datetime\" nil=\"true\" />\n";       

        xml += ( this.completedAt.mktime() > 0 ) ?
                "\t<completed-at type=\"datetime\" >%s</completed-at>\n".printf(this.completedAt.format("%FT00:00:00%z")) :
                "\t<completed-at type=\"datetime\" nil=\"true\" />\n";       
 
        xml += ( this.projectID > 0 ) ? 
                "\t<project-id type=\"integer\">%d</project-id>\n".printf(this.projectID) :
                "\t<project-id type=\"integer\" nil=\"true\"/>\n";              


        xml += ( this.text != "" ) ?
                "\t<notes>%s</notes>\n".printf(this.text) :
                "";
//        xml += ( this.id > 0 ) ? 
//                "\t<id type=\"integer\">%d</id>\n".printf(this.id) :
//                "";
         
        xml += "</todo>\n";
        return xml;
    }
}
