[CCode (cprefix = "Couchdb", lower_case_cprefix = "couchdb_")]
namespace CouchDB {

	[CCode (cheader_filename = "couchdb-glib.h")]
	public class Session : GLib.Object {
		[CCode (has_construct_function = false)]
		public Session (string? hostname = null);

		public unowned string get_uri ();

		public GLib.SList<string> list_databases () throws GLib.Error;
		public GLib.SList<CouchDB.DocumentInfo> list_documents (string dbname) throws GLib.Error;

		public CouchDB.DatabaseInfo get_database_info (string dbname) throws GLib.Error;
		public bool create_database (string dbname) throws GLib.Error;
		public bool delete_database (string dbname) throws GLib.Error;

        public void listen_for_changes (char *dbname);

		public void enable_authentication (CouchDB.Credentials credentials);
		public void disable_authentication ();
		public bool is_authentication_enabled ();

		[CCode (cname = "couchdb_document_get")]
		public CouchDB.Document get_document (string dbname, string docid) throws GLib.Error;

		public virtual signal void authentication_failed ();
	}

	[CCode (cheader_filename = "couchdb-glib.h")]
	public class Credentials : GLib.Object {
		[CCode (has_construct_function = false)]
		public Credentials.with_oauth (string consumer_key, string onsumer_secret, string token_key, string token_secret);
		public Credentials.with_username_and_password (string username, string password);
		CouchDB.CredentialsType get_auth_type();
    }

    [CCode (cprefix="COUCHDB_CREDENTIALS_TYPE_")]
    public enum CredentialsType {
 	    UNKNOWN,
	    OAUTH,
	    USERNAME_AND_PASSWORD   
    }

	[CCode (cheader_filename = "couchdb-glib.h")]
	public class DatabaseInfo {
		[CCode (has_construct_function = false)]
		public 	DatabaseInfo (string dbname, int doc_count, int doc_del_count, int update_seq, bool compact_running, int disk_size);
		public unowned string get_dbname ();
		public int get_documents_count ();
		public int get_deleted_documents_count ();
		public int get_update_sequence ();
		public bool is_compact_running ();
		public int get_disk_size ();
	}

	[CCode (cheader_filename = "couchdb-glib.h")]
	public class Document : GLib.Object {
		[CCode (has_construct_function = false)]
		public Document (CouchDB.Session couchdb);
		[CCode (has_construct_function = false)]
		[CCode (cname = "couchdb_document_contact_new")]
		public Document.contact (CouchDB.Session couchdb);

		public bool put (string dbname) throws GLib.Error;
		public bool delete () throws GLib.Error;

		public unowned string get_id ();
		public void set_id (string id);
		public unowned string get_revision ();
		public void set_revision (string revision);
		public unowned string get_record_type ();
		public void set_record_type (string record_type);

		public bool has_field (string field);
		public void remove_field (string field);

		public bool get_boolean_field (string field);
		public void set_boolean_field (string field, bool @value);
		public int get_int_field (string field);
		public void set_int_field (string field, int @value);
		public double get_double_field (string field);
		public void set_double_field (string field, double @value);
		public unowned string? get_string_field (string field);
		public void set_string_field (string field, string? @value);

		public CouchDB.StructField? get_struct_field (string field);
		public void set_struct_field (string field, CouchDB.StructField @value);

		public CouchDB.StructField? get_application_annotations ();
		public void set_application_annotations (CouchDB.StructField? annotations);

		public string? to_string ();
		public Json.Object get_json_object ();

        public string contact_get_first_name ();
        public void contact_set_first_name (string first_name);
        public string contact_get_last_name ();
        public void contact_set_last_name (string last_name);
        public string contact_get_nick_name ();
        public void contact_set_nick_name (string nick_name);
        public string contact_get_spouse_name ();
        public void contact_set_spouse_name (string spouse_name);
        public string contact_get_birth_date ();
        public void contact_set_birth_date (string birth_date);
        public string contact_get_wedding_date ();
        public void contact_set_wedding_date (string wedding_date);
        public string contact_get_company ();
        public void contact_set_company (string company);
        public string contact_get_department ();
        public void contact_set_department (string department);
        public string contact_get_title ();
        public void contact_set_title (string title);
        public string contact_get_job_title ();
        public void contact_set_job_title (string job_title);
        public string contact_get_manager_name ();
        public void contact_set_manager_name (string manager_name);
        public string contact_get_assistant_name ();
        public void contact_set_assistant_name (string assistant_name);
        public string contact_get_office ();
        public void contact_set_office (string office);
        public string contact_get_notes ();
        public void contact_set_notes (string notes);
        public GLib.SList<string> contact_get_email_addresses ();
		public void contact_set_email_addresses (GLib.SList<string> list);
		public GLib.SList<string> contact_get_phone_numbers ();
		public void contact_set_phone_numbers (GLib.SList<string> list);
		public GLib.SList<string> contact_get_addresses ();
		public void contact_set_addresses (GLib.SList<string> list);
		public GLib.SList<string> contact_get_im_addresses ();
		public void contact_set_im_addresses (GLib.SList<string> list);
		public GLib.SList<string> contact_get_urls ();
		public void contact_set_urls (GLib.SList<string> list);
	}

	[CCode (cheader_filename = "couchdb-glib.h")]
	public class DocumentInfo {
		[CCode (has_construct_function = false)]
		public DocumentInfo (string docid, string revision);
		public unowned string get_docid ();
		public unowned string get_revision ();
	}

	[CCode (cheader_filename = "couchdb-glib.h")]
	public class StructField {
		[CCode (has_construct_function = false)]
		public StructField ();
		[CCode (has_construct_function = false)]
		public StructField.from_string (string str);
		[CCode (has_construct_function = false)]
		public StructField.from_json_object (Json.Object json_object);
		
		public bool has_field (string field);
		public void remove_field (string field);

		public bool get_boolean_field (string field);
		public void set_boolean_field (string field, bool @value);
		public int get_int_field (string field);
		public void set_int_field (string field, int @value);
		public double get_double_field (string field);
		public void set_double_field (string field, double @value);
		public unowned string? get_string_field (string field);
		public void set_string_field (string field, string? @value);
		public CouchDB.StructField get_struct_field (string field);
		public void set_struct_field (string field, CouchDB.StructField @value);

		public unowned string? get_uuid ();
		public void set_uuid (string? uuid);
		public string to_string ();
		public Json.Object get_json_object();
	}
}

